#!/usr/bin/env python

""" MultiQC submodule to parse output from RSeQC junction_saturation.py
http://rseqc.sourceforge.net/#junction-saturation-py """

from collections import OrderedDict
import logging
import re

from multiqc.plots import linegraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """ Find RSeQC junction_saturation frequency reports and parse their data """

    # Set up vars
    self.junction_saturation_all = dict()
    self.junction_saturation_known = dict()
    self.junction_saturation_novel = dict()

    # Go through files and parse data
    for f in self.find_log_files('rseqc/junction_saturation'):
        parsed = dict()
        for l in f['f'].splitlines():
            r = re.search(r"^([xyzw])=c\(([\d,]+)\)$", l)
            if r:
                parsed[r.group(1)] = [float(i) for i in r.group(2).split(',')]
        if len(parsed) == 4:
            if parsed['z'][-1] == 0:
                log.warning("Junction saturation data all zeroes, skipping: '{}'".format(f['s_name']))
            else:
                if f['s_name'] in self.junction_saturation_all:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
                self.add_data_source(f, section='junction_saturation')
                self.junction_saturation_all[f['s_name']] = OrderedDict()
                self.junction_saturation_known[f['s_name']] = OrderedDict()
                self.junction_saturation_novel[f['s_name']] = OrderedDict()
                for k, v in enumerate(parsed['x']):
                    self.junction_saturation_all[f['s_name']][v] = parsed['z'][k]
                    self.junction_saturation_known[f['s_name']][v] = parsed['y'][k]
                    self.junction_saturation_novel[f['s_name']][v] = parsed['w'][k]

    # Filter to strip out ignored sample names
    self.junction_saturation_all = self.ignore_samples(self.junction_saturation_all)
    self.junction_saturation_known = self.ignore_samples(self.junction_saturation_known)
    self.junction_saturation_novel = self.ignore_samples(self.junction_saturation_novel)

    if len(self.junction_saturation_all) > 0:

        # Add line graph to section
        pconfig = {
            'id': 'rseqc_junction_saturation_plot',
            'title': 'RSeQC: Junction Saturation',
            'ylab': 'Number of Junctions',
            'ymin': 0,
            'xlab': "Percent of reads",
            'xmin': 0,
            'xmax': 100,
            'tt_label': "<strong>{point.x}% of reads</strong>: {point.y:.2f}",
            'data_labels': [
                {'name': 'All Junctions'},
                {'name': 'Known Junctions'},
                {'name': 'Novel Junctions'}
            ],
            'cursor': 'pointer',
            'click_func': 'single_sample_plot'
        }
        self.add_section (
            name = 'Junction Saturation',
            anchor = 'rseqc-junction_saturation',
            description = '''<a href="http://rseqc.sourceforge.net/#junction-saturation-py" target="_blank">Junction Saturation</a>
                counts the number of known splicing junctions that are observed
                in each dataset. If sequencing depth is sufficient, all (annotated) splice junctions should
                be rediscovered, resulting in a curve that reaches a plateau. Missing low abundance splice
                junctions can affect downstream analysis.</p>
                <div class="alert alert-info" id="rseqc-junction_sat_single_hint">
                  <span class="glyphicon glyphicon-hand-up"></span>
                  Click a line to see the data side by side (as in the original RSeQC plot).
                </div><p>''',
            plot = linegraph.plot([
                    self.junction_saturation_all,
                    self.junction_saturation_known,
                    self.junction_saturation_novel
                ], pconfig)
        )

    # Return number of samples found
    return len(self.junction_saturation_all)



from __future__ import print_function
from io import open
import argparse
import logging
import re
import base64
import hashlib
from bs4 import BeautifulSoup

logger = logging.getLogger(__name__)

parser = argparse.ArgumentParser(description='Checks inline scripts in report.html against a ' \
                                             'whitelist of hashes (Content Security Policy)')

parser.add_argument('--report', help='Report file (.html)', required=True)
parser.add_argument('--whitelist', help='Whitelist to compare against', required=True)
args = parser.parse_args()

def is_executable(script):
    executable_types = {
        'text/javascript',
        'application/javascript',
        'module'
        'text/ecmascript',
        'application/ecmascript',
    }
    script_type = script.attrs.get('type', 'text/javascript')
    return script_type in executable_types

def get_hash(script):
    return 'sha256-' + base64.b64encode(hashlib.sha256(script.encode('utf-8')).digest()).decode()

whitelist_with_comments = open(args.whitelist, 'r', encoding='utf-8').read()
whitelist = re.sub(r'#.*', '', whitelist_with_comments)

html_report = open(args.report, 'r', encoding='utf-8').read()
soup = BeautifulSoup(html_report, features="html.parser")

scripts = [script.get_text() for script in soup.select('script') if is_executable(script)]
missing_scripts = [script for script in scripts if get_hash(script) not in whitelist]

if missing_scripts:
    logger.warning('The following scripts are missing from {}'.format(args.whitelist))
    for script in missing_scripts:
        hash = get_hash(script)
        snippet = script.replace('\n', '')[0:80]
        logger.warning("  '{}' # {}".format(hash, snippet))
    exit(1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" MultiQC module to parse output from phantompeakqualtools """

from __future__ import print_function
from collections import OrderedDict
import logging

from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='phantompeakqualtools', anchor='phantompeakqualtools',
        href='https://www.encodeproject.org/software/phantompeakqualtools',
        info="computes informative enrichment and quality measures for ChIP-seq/DNase-seq/FAIRE-seq/MNase-seq data.")

        # Parse logs
        self.phantompeakqualtools_data = dict()
        for f in self.find_log_files('phantompeakqualtools/out', filehandles=False):
            self.parse_phantompeakqualtools(f)

        # Filter to strip out ignored sample names
        self.phantompeakqualtools_data  =  self.ignore_samples(self.phantompeakqualtools_data)

        # Warning when no files are found
        if len(self.phantompeakqualtools_data) == 0:
            raise UserWarning

        # Log
        log.info("Found {} logs".format(len(self.phantompeakqualtools_data)))

        # Write parsed data to a file
        self.write_data_file(self.phantompeakqualtools_data, 'multiqc_phantompeakqualtools')

        # Report section
        self.phantompeakqualtools_general_stats()

    # Parse spp.out file from phantompeakqualtools
    def parse_phantompeakqualtools(self, f):
        s_name = self.clean_s_name(f['s_name'], f['root'])
        parsed_data = {}
        lines = f['f'].splitlines()
        for l in lines:
            s = l.split("\t")
            parsed_data['Estimated_Fragment_Length_bp'] = int(s[2].split(",")[0])
            parsed_data['NSC'] = float(s[8])
            parsed_data['RSC'] = float(s[9])
        if len(parsed_data) > 0:
            if s_name in self.phantompeakqualtools_data:
                log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
            self.add_data_source(f, s_name)
            self.phantompeakqualtools_data[s_name] = parsed_data

    # Report fragment length, NSC and RSC in general stat table
    def phantompeakqualtools_general_stats(self):
        """ Add columns to General Statistics table """
        headers = OrderedDict()
        headers['Estimated_Fragment_Length_bp'] = {
            'title': 'Frag Length',
            'description': 'Estimated fragment length (bp)',
            'min': 0,
            'format': '{:,.0f}'
        }
        headers['NSC'] = {
            'title': 'NSC',
            'description': 'Normalized strand cross-correlation',
            'max': 10,
            'min': 0,
            'format': '{:,.2f}',
            'scale': 'RdYlGn-rev'
        }
        headers['RSC'] = {
            'title': 'RSC',
            'description': 'Relative strand cross-correlation',
            'max': 10,
            'min': 0,
            'format': '{:,.2f}',
            'scale': 'RdYlBu-rev'
        }
        self.general_stats_addcols(self.phantompeakqualtools_data, headers)



#!/usr/bin/env python

""" MultiQC submodule to parse output from Rockhopper summary files
https://cs.wellesley.edu/~btjaden/Rockhopper/ """

from __future__ import print_function
import logging
import re
from collections import OrderedDict

from multiqc import config
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):

        # Initialize the parent object
        super(MultiqcModule, self).__init__(
            name = 'Rockhopper',
            anchor = 'rockhopper',
            href = 'https://cs.wellesley.edu/~btjaden/Rockhopper/',
            info = """
            is a comprehensive and user-friendly system
            for computational analysis of bacterial RNA-seq data.
            It can align reads to genomes, assemble transcripts,
            identify transcript boundaries, and discover novel
            transcripts such as small RNAs.
            """
        )

        # Set up vars
        self.rh_data = dict()

        # Parse summary file
        for f in self.find_log_files('rockhopper'):
            self.parse_rockhopper_summary(f)

        # Filter to strip out ignored sample names
        self.rh_data = self.ignore_samples(self.rh_data)

        if len(self.rh_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.rh_data)))

        # Write to file
        self.write_data_file(self.rh_data, 'multiqc_rockhopper')

        # Basic stats Table
        self.rockhopper_general_stats_table()

        # Alignment bar plot
        self.rockhopper_count_bar_plot()


    def parse_rockhopper_summary(self,f):

        s_name = None

        # Initialize stats fields
        stats_index = [
            'mRNA-sense',
            'mRNA-antisense',
            'rRNA-sense',
            'rRNA-antisense',
            'tRNA-sense',
            'tRNA-antisense',
            'ncRNA-sense',
            'ncRNA-antisense',
            'unannotated'
        ]

        results = { name:0 for name in stats_index }

        # Parse rockhopper output line-by-line since there may be many genomes
        lines = f['f'].split('\n')
        for i, line in enumerate(lines):

            # Get sample name
            if line.startswith('Aligning sequencing reads from file:'):
                s_name = line.split(':', 1)[1].strip()
                s_name = self.clean_s_name(s_name,f['root'])

            elif line.startswith('Aligning sequencing reads from files:'):
                s_name = lines[i+1].strip()
                s_name = self.clean_s_name(s_name,f['root'])

            # Get total number of reads read by rockhopper
            if line.startswith('Total reads:'):
                results['total-reads'] = int(re.search('Total reads:\s*(\d*)',line).group(1))
                i += 1

            # Get number of reads aligned to each genome
            elif line.startswith('Successfully aligned reads'):

                # Get Genome ID
                genome = re.search('\(>(.*?) .*\)',line).group(1)

                # Get number of aligned reads
                genome_reads = int(re.search('Successfully aligned reads:\s*(\d*)',line).group(1))

                # Get percent of reads in each category
                stats = [int(re.search('(\d+)\%',subline).group(1)) for subline in lines[i+1:i+10]]
                for name,val in zip(stats_index,stats):
                    # Convert percentages to true number of reads in each category
                    results[name] += int(round(val*genome_reads/100))

                # Skip 10 lines
                i += 10

            else:
                i += 1

        if len(results) > 0 and s_name is not None:

            # Calculate unaligned read count
            total_mapped_reads  = sum([v for k,v in results.items() if k != 'total-reads'])
            results['unaligned'] = results['total-reads'] - total_mapped_reads

            if s_name in self.rh_data:
                log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
            self.add_data_source(f, s_name)
            self.rh_data[s_name] = results

    def rockhopper_general_stats_table(self):
        """ Take the parsed stats from the Rockhopper summary and add it to the
        basic stats table at the top of the report """

        headers = {}
        headers['mRNA-sense'] = {
            'title': 'CDS Reads ({})'.format(config.read_count_prefix),
            'description':'Reads aligned to coding regions ({})'.format(config.read_count_desc),
            'min':0,
            'scale':'Blues',
            'modify': lambda x: x * config.read_count_multiplier,
            'shared_key': 'read_count'
        }
        headers['mRNA-antisense'] = {
            'title': 'CDS Reads (a/s, {})'.format(config.read_count_prefix),
            'description':'Antisense reads aligned to coding regions ({})'.format(config.read_count_desc),
            'min':0,
            'scale':'Blues',
            'modify': lambda x: x * config.read_count_multiplier,
            'shared_key': 'read_count',
            'hidden':True
        }
        headers['rRNA-sense'] = {
            'title': 'rRNA Reads ({})'.format(config.read_count_prefix),
            'description':'Reads aligned to rRNA ({})'.format(config.read_count_desc),
            'min':0,
            'scale':'Blues',
            'modify': lambda x: x * config.read_count_multiplier,
            'shared_key': 'read_count'
        }
        headers['rRNA-antisense'] = {
            'title': 'rRNA Reads (a/s, {})'.format(config.read_count_prefix),
            'description':'Antisense reads aligned to rRNA ({})'.format(config.read_count_desc),
            'min':0,
            'scale':'Blues',
            'modify': lambda x: x * config.read_count_multiplier,
            'shared_key': 'read_count',
            'hidden':True
        }
        self.general_stats_addcols(self.rh_data,headers)

    def rockhopper_count_bar_plot(self):
        """ Stacked bar plot showing counts of reads """

        pconfig = {
            'id':'rockhopper_reads_counts_plot',
            'title': 'Rockhopper: Alignment types',
            'ylab': 'Number of reads',
            'tt_percentage': False
        }

        # Plot bar graph of groups
        keys = OrderedDict()
        keys['mRNA-sense']      = {'name': "mRNA (Sense)"}
        keys['mRNA-antisense']  = {'name': "mRNA (Antisense)"}
        keys['rRNA-sense']      = {'name': "rRNA (Sense)"}
        keys['rRNA-antisense']  = {'name': "rRNA (Antisense)"}
        keys['tRNA-sense']      = {'name': "tRNA (Sense)"}
        keys['tRNA-antisense']  = {'name': "tRNA (Antisense)"}
        keys['ncRNA-sense']     = {'name': "ncRNA (Sense)"}
        keys['ncRNA-antisense'] = {'name': "ncRNA (Antisense)"}
        keys['unannotated']     = {'name': "Unannotated"}
        keys['unaligned']       = {'name':'Unaligned'}

        self.add_section (
            name = 'Rockhopper',
            anchor = 'rockhopper_reads_counts',
            description = """
            This plot shows the number of reads mapped to
            different regions of the genome, accounting for
            sense and antisense alignment, if relevant.
            """,
            plot = bargraph.plot(self.rh_data, keys, pconfig)
        )



#!/usr/bin/env python

""" MultiQC code to export data to MegaQC / flat JSON files """

from __future__ import print_function
import gzip
import io
import json
import os
import requests

from multiqc import config
log = config.logger

# Custom encoder to handle lambda functions
class MQCJSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if callable(obj):
            try:
                return obj(1)
            except:
                return None
        return json.JSONEncoder.default(self, obj)

def multiqc_dump_json(report):
    exported_data = dict()
    export_vars = {
        'report': [
            'data_sources',
            'general_stats_data',
            'general_stats_headers',
            'multiqc_command',
            'plot_data',
            'saved_raw_data',
        ],
        'config': [
            'analysis_dir',
            'creation_date',
            'git_hash',
            'intro_text',
            'report_comment',
            'report_header_info',
            'script_path',
            'short_version',
            'subtitle',
            'title',
            'version',
        ]
    }
    for s in export_vars:
        for k in export_vars[s]:
            try:
                if s == 'config':
                    d = {'{}_{}'.format(s, k): getattr(config, k)}
                elif s == 'report':
                    d = {'{}_{}'.format(s, k): getattr(report, k)}
                json.dumps(d, cls=MQCJSONEncoder, ensure_ascii=False) # Test that exporting to JSON works
                exported_data.update(d)
            except (TypeError, KeyError, AttributeError):
                log.warning("Couldn't export data key '{}.{}'".format(s, k))
        # Get the absolute paths of analysis directories
        exported_data['config_analysis_dir_abs'] = list()
        for d in exported_data.get('config_analysis_dir', []):
            try:
                exported_data['config_analysis_dir_abs'].append(os.path.abspath(d))
            except:
                pass
    return exported_data


def multiqc_api_post(exported_data):
    headers = { 'Content-Type': 'application/json', 'content-encoding': 'gzip' }
    if config.megaqc_access_token is not None:
        headers['access_token'] = config.megaqc_access_token
    post_data = json.dumps({'data': exported_data}, cls=MQCJSONEncoder, ensure_ascii=False, indent=2)
    post_data = post_data.encode('utf-8', 'ignore')

    # Gzip the JSON for massively decreased filesize
    sio_obj = io.BytesIO()
    gzfh = gzip.GzipFile(fileobj=sio_obj, mode='w')
    gzfh.write(post_data)
    gzfh.close()
    request_body = sio_obj.getvalue()

    log.debug("Sending data to MegaQC")
    log.debug("MegaQC URL: {}".format(config.megaqc_url))
    try:
        r = requests.post(config.megaqc_url, headers=headers, data=request_body, timeout=config.megaqc_timeout)
    except (requests.exceptions.ConnectTimeout, requests.exceptions.ReadTimeout) as e:
        log.error("Timed out when sending data: {}".format(e))
    except requests.exceptions.ConnectionError:
        log.error("Couldn't connect to MegaQC URL {}".format(config.megaqc_url))
    except Exception as e:
        log.error("Error sending data: {}".format(e))
    else:
        try:
            api_r = json.loads(r.text)
        except Exception as e:
            log.error('Error: JSON response could not be parsed (status code: {})'.format(r.status_code))
            return None
        if r.status_code == 200:
            if api_r['success']:
                log.info('{}'.format(api_r['message']))
            else:
                log.error('Error - {}'.format(api_r['message']))
        else:
            if r.status_code == 403:
                if config.megaqc_access_token is not None:
                    log.error('Error 403: Authentication error, megaqc_access_token not recognised')
                else:
                    log.error('Error 403: Authentication error, megaqc_access_token is required')
            else:
                log.debug("MegaQC API status code was {}".format(r.status_code))
                log.error('Error - {}'.format(api_r.get('message', 'Unknown problem')))



#!/usr/bin/env python

""" MultiQC lint helpers. Simple additional tests to run when
--lint is specified (outside scope of normal functions) """

from __future__ import print_function
import os
import yaml

from multiqc.utils import config, report
logger = config.logger

def run_tests():
    """ Run all lint tests """
    if config.lint:
        check_mods_docs_readme()


def check_mods_docs_readme():
    """ Check that all modules are listed in the YAML index
    at the top of docs/README.md """

    docs_mods = []

    readme_fn = os.path.join( os.path.dirname(config.MULTIQC_DIR), 'docs', 'README.md')
    if not os.path.isfile(readme_fn):
        if os.environ.get('GITHUB_WORKSPACE') is not None:
            readme_fn = os.path.join( os.environ.get('GITHUB_WORKSPACE'), 'MultiQC', 'docs', 'README.md')
        if not os.path.isfile(readme_fn):
            logger.warning("Can't check docs readme in lint test as file doesn't exist: {}".format(readme_fn))
            return None
    with open(readme_fn) as f:
        fm = next(yaml.load_all(f, Loader=yaml.SafeLoader))

    for section in fm['MultiQC Modules']:
        for name, fn in fm['MultiQC Modules'][section].items():
            # remove modules/ and .md
            docs_mods.append(fn[8:-3])

    # Check that installed modules are listed in docs/README.md
    for m in config.avail_modules.keys():
        if m not in docs_mods and m != 'custom_content':
            errmsg = "LINT: Module '{}' found in installed modules, but not docs/README.md".format(m)
            logger.error(errmsg)
            report.lint_errors.append(errmsg)

    # Check that modules in docs/README.md are installed
    for m in docs_mods:
        if m not in config.avail_modules.keys() and m != 'custom_content':
            errmsg = "LINT: Module '{}' found in docs/README.md, but not installed modules".format(m)
            logger.error(errmsg)
            report.lint_errors.append(errmsg)



#!/usr/bin/env python

""" MultiQC module to parse output from MACS2 """

from __future__ import print_function
from collections import OrderedDict
import logging
import re

from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='MACS2', anchor='macs',
        href='https://github.com/taoliu/MACS',
        info="identifies transcription factor binding sites in ChIP-seq data.")

        # Parse logs
        self.macs_data = dict()
        for f in self.find_log_files('macs2', filehandles=True):
            self.parse_macs(f)

        # Filter to strip out ignored sample names
        self.macs_data = self.ignore_samples(self.macs_data)

        if len(self.macs_data) == 0:
            raise UserWarning

        log.info("Found {} logs".format(len(self.macs_data)))
        self.write_data_file(self.macs_data, 'multiqc_macs')

        self.macs_general_stats()
        self.macs_filtered_reads_plot()


    def parse_macs(self, f):
        regexes = {
            'name': r"# name = (.+)$",
            'fragment_size': r"# (?:fragment|tag) size is determined as (\d+) bps",
            'treatment_fragments_total': r"# total (?:fragments|tags) in treatment: (\d+)",
            'treatment_fragments_after_filtering': r"# (?:fragments|tags) after filtering in treatment: (\d+)",
            'treatment_max_duplicates': r"# maximum duplicate (?:fragments|tags at the same position) in treatment = (\d+)",
            'treatment_redundant_rate': r"# Redundant rate in treatment: ([\d\.]+)",
            'control_fragments_total': r"# total (?:fragments|tags) in control: (\d+)",
            'control_fragments_after_filtering': r"# (?:fragments|tags) after filtering in control: (\d+)",
            'control_max_duplicates': r"# maximum duplicate (?:fragments|tags at the same position) in control = (\d+)",
            'control_redundant_rate': r"# Redundant rate in control: ([\d\.]+)",
            'd': r"# d = (\d+)",
        }
        s_name = f['s_name']
        parsed_data = dict()
        for l in f['f']:
            for k, r in regexes.items():
                match = re.search(r, l)
                if match:
                    if k == 'name':
                        s_name = self.clean_s_name(match.group(1).strip(), f['root'])
                    else:
                        parsed_data[k] = float(match.group(1).strip())
            if not l.startswith('#') and l.strip():
                break
        if len(parsed_data) > 0:
            if s_name in self.macs_data:
                log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
            self.macs_data[s_name] = parsed_data

    def macs_general_stats(self):
        """ Add columns to General Statistics table """
        headers = OrderedDict()
        headers['d'] = {
            'title': 'Fragment Length',
            'min': 0,
            'format': '{:,.0f}'
        }
        headers['treatment_redundant_rate'] = {
            'title': 'Treatment Redundancy',
            'description': 'Redundant rate in treatment',
            'max': 1,
            'min': 0,
            'format': '{:,.2f}',
            'scale': 'RdYlBu-rev'
        }
        headers['control_redundant_rate'] = {
            'title': 'Control Redundancy',
            'description': 'Redundant rate in control',
            'max': 1,
            'min': 0,
            'format': '{:,.2f}',
            'scale': 'RdYlBu-rev'
        }
        self.general_stats_addcols(self.macs_data, headers)

    def macs_filtered_reads_plot(self):
        """ Plot of filtered reads for control and treatment samples """
        data = dict()
        req_cats = ['control_fragments_total', 'control_fragments_after_filtering', 'treatment_fragments_total', 'treatment_fragments_after_filtering']
        for s_name, d in self.macs_data.items():
            if all([c in d for c in req_cats]):
                data['{}: Control'.format(s_name)] = dict()
                data['{}: Treatment'.format(s_name)] = dict()
                data['{}: Control'.format(s_name)]['fragments_filtered'] = d['control_fragments_total'] - d['control_fragments_after_filtering']
                data['{}: Control'.format(s_name)]['fragments_not_filtered'] = d['control_fragments_after_filtering']
                data['{}: Treatment'.format(s_name)]['fragments_filtered'] = d['treatment_fragments_total'] - d['treatment_fragments_after_filtering']
                data['{}: Treatment'.format(s_name)]['fragments_not_filtered'] = d['treatment_fragments_after_filtering']

        # Check that we have something to plot
        if len(data) == 0:
            return

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['fragments_not_filtered'] = { 'color': '#437BB1', 'name': 'Remaining fragments' }
        keys['fragments_filtered'] =     { 'color': '#B1084C', 'name': 'Filtered fragments' }

        # Config for the plot
        pconfig = {
            'id': 'macs2_filtered',
            'title': 'MACS2: Filtered Fragments',
            'ylab': '# Fragments',
            'cpswitch_counts_label': 'Number of Fragments',
            'hide_zero_cats': False
        }

        self.add_section(
            plot = bargraph.plot(data, keys, pconfig)
        )



#!/usr/bin/env python

""" MultiQC submodule to parse output from RSeQC read_duplication.py
http://rseqc.sourceforge.net/#read-duplication-py """

from collections import OrderedDict
import logging

from multiqc.plots import linegraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """ Find RSeQC read_duplication reports and parse their data """

    # Set up vars
    self.read_dups = dict()

    # Go through files and parse data
    for f in self.find_log_files('rseqc/read_duplication_pos'):
        if f['f'].startswith('Occurrence	UniqReadNumber'):
            if f['s_name'] in self.read_dups:
                log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
            self.add_data_source(f, section='read_duplication')
            self.read_dups[f['s_name']] = OrderedDict()
            for l in f['f'].splitlines():
                s = l.split()
                try:
                    if int(s[0]) <= 500:
                        self.read_dups[f['s_name']][int(s[0])] = int(s[1])
                except:
                    pass

    # Filter to strip out ignored sample names
    self.read_dups = self.ignore_samples(self.read_dups)

    if len(self.read_dups) > 0:

        # Add line graph to section
        pconfig = {
            'smooth_points': 200,
            'id': 'rseqc_read_dups_plot',
            'title': 'RSeQC: Read Duplication',
            'ylab': 'Number of Reads (log10)',
            'xlab': "Occurrence of read",
            'yLog': True,
            'tt_label': "<strong>{point.x} occurrences</strong>: {point.y} reads",
        }

        self.add_section (
            name = 'Read Duplication',
            anchor = 'rseqc-read_dups',
            description = '<a href="http://rseqc.sourceforge.net/#read-duplication-py" target="_blank">read_duplication.py</a>' \
                " calculates how many alignment positions have a certain number of exact duplicates."\
                " Note - plot truncated at 500 occurrences and binned.</p>",
            plot = linegraph.plot(self.read_dups, pconfig)
        )

    # Return number of samples found
    return len(self.read_dups)






#!/usr/bin/env python

""" MultiQC submodule to parse output from deepTools plotProfile """

import logging

from multiqc.plots import linegraph

# Initialise the logger
log = logging.getLogger(__name__)

class plotProfileMixin():
    def parse_plotProfile(self):
        """Find plotProfile output"""
        self.deeptools_plotProfile = dict()
        for f in self.find_log_files('deeptools/plotProfile', filehandles=False):
            parsed_data, bin_labels, converted_bin_labels = self.parsePlotProfileData(f)
            for k, v in parsed_data.items():
                if k in self.deeptools_plotProfile:
                    log.warning("Replacing duplicate sample {}.".format(k))
                self.deeptools_plotProfile[k] = v
            if len(parsed_data) > 0:
                self.add_data_source(f, section='plotProfile')

        self.deeptools_plotProfile = self.ignore_samples(self.deeptools_plotProfile)

        if len(self.deeptools_plotProfile) > 0:

            # Try to do plot bands but don't crash if the labels aren't as we expect
            xPlotBands = []
            xPlotLines = []
            plotBandHelp = ''
            try:
                xPlotBands.append({'from': converted_bin_labels[bin_labels.index('TES')], 'to': converted_bin_labels[-1], 'color': '#f7cfcf'})
                xPlotBands.append({'from': converted_bin_labels[bin_labels.index('TSS')], 'to': converted_bin_labels[bin_labels.index('TES')], 'color': '#ffffe2'})
                xPlotBands.append({'from': converted_bin_labels[0], 'to': converted_bin_labels[bin_labels.index('TSS')], 'color': '#e5fce0'})
                xPlotLines.append({'width': 1, 'value': converted_bin_labels[bin_labels.index('TES')], 'dashStyle': 'Dash', 'color': '#000000'})
                xPlotLines.append({'width': 1, 'value': converted_bin_labels[bin_labels.index('TSS')], 'dashStyle': 'Dash', 'color': '#000000'})
                plotBandHelp = """
                    * Green: {} upstream of gene to {}
                    * Yellow: {} to {}
                    * Pink: {} to {} downstream of gene
                    """.format(
                        list(filter(None,bin_labels))[0],
                        list(filter(None,bin_labels))[1],
                        list(filter(None,bin_labels))[1],
                        list(filter(None,bin_labels))[2],
                        list(filter(None,bin_labels))[2],
                        list(filter(None,bin_labels))[3]
                    )
            except ValueError:
                pass


            config = {
                'id': 'read_distribution_profile',
                'title': 'deeptools: Read Distribution Profile after Annotation',
                'ylab': 'Occurrence',
                'xlab': None,
                'smooth_points': 100,
                'xPlotBands': xPlotBands,
                'xPlotLines': xPlotLines
            }

            self.add_section (
                name = 'Read Distribution Profile after Annotation',
                anchor = 'read_distribution_profile_plot',
                description = """
                    Accumulated view of the distribution of sequence reads related to the closest annotated gene.
                    All annotated genes have been normalized to the same size.

                    {}""".format(plotBandHelp),
                plot=linegraph.plot(self.deeptools_plotProfile, config)
            )

        return len(self.deeptools_plotProfile)

    def parsePlotProfileData(self, f):
        d = dict()
        bin_labels = []
        bins = []
        for line in f['f'].splitlines():
            cols = line.rstrip().split("\t")
            if cols[0] == "bin labels":
                for col in cols[2:len(cols)]:
                    if col not in list(filter(None,bin_labels)):
                        bin_labels.append(col)
                    else:
                        break
            elif cols[0] == "bins":
                for col in cols[2:len(cols)]:
                    if len(bins)!=len(bin_labels):
                        bins.append(self._int(col))
                    else:
                        break
            else:
                s_name = self.clean_s_name(cols[0], f['root'])
                d[s_name] = dict()

                factors = {'Kb': 1e3, 'Mb': 1e6, 'Gb': 1e9}
                convert_factor = 1
                for k, v in factors.items():
                    if k in bin_labels[0]:
                        convert_factor *= v
                        start = float(bin_labels[0].strip(k)) * convert_factor
                step = self._int(abs(start/bin_labels.index('TSS')))
                end = step*(len(bin_labels)-bin_labels.index('TSS')-1)
                converted_bin_labels = range((self._int(start)+step), (self._int(end)+step), step)

                for i in bins:
                    d[s_name].update({converted_bin_labels[i-1]:float(cols[i+1])})

        return d, bin_labels, converted_bin_labels



#!/usr/bin/env python

""" MultiQC module to parse output from HISAT2 """

from __future__ import print_function
from collections import OrderedDict
import logging
import re

from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """ HISAT2 module, parses stderr logs. """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name="HISAT2", anchor="hisat2",
        href='https://ccb.jhu.edu/software/hisat2/',
        info="is a fast and sensitive alignment program for mapping "\
             "NGS reads (both DNA and RNA) against a reference genome or "\
             "population of reference genomes.")

        # Find and load any HISAT2 reports
        self.hisat2_data = dict()
        for f in self.find_log_files('hisat2', filehandles=True):
            self.parse_hisat2_logs(f)

        # Filter to strip out ignored sample names
        self.hisat2_data = self.ignore_samples(self.hisat2_data)

        if len(self.hisat2_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.hisat2_data)))

        # Write parsed report data to a file
        self.write_data_file(self.hisat2_data, 'multiqc_hisat2')

        # Basic Stats Table
        # Report table is immutable, so just updating it works
        self.hisat2_general_stats_table()

        # Alignment Rate Plot
        self.hisat2_alignment_plot()


    def parse_hisat2_logs(self, f):
        """
        Parse statistics generated by HISAT2 >= v2.1.0 that has been run
        with the --new-summary option. Older versions or logs from runs without
        that option are identical to that from bowtie2 and will be parsed
        by that module.
        """

        # Regexes
        regexes = {
            'unpaired_total': r"Total(?: unpaired)? reads: (\d+)",
            'unpaired_aligned_none': r"Aligned 0 times?: (\d+) \([\d\.]+%\)",
            'unpaired_aligned_one': r"Aligned 1 time: (\d+) \([\d\.]+%\)",
            'unpaired_aligned_multi': r"Aligned >1 times: (\d+) \([\d\.]+%\)",
    		'paired_total': r"Total pairs: (\d+)",
    		'paired_aligned_none': r"Aligned concordantly or discordantly 0 time: (\d+) \([\d\.]+%\)",
    		'paired_aligned_one': r"Aligned concordantly 1 time: (\d+) \([\d\.]+%\)",
    		'paired_aligned_multi': r"Aligned concordantly >1 times: (\d+) \([\d\.]+%\)",
    		'paired_aligned_discord_one': r"Aligned discordantly 1 time: (\d+) \([\d\.]+%\)",
        }

        # Go through log file line by line
        s_name = f['s_name']
        parsed_data = {}

        for l in f['f']:

            # Attempt in vain to find original hisat2 command, logged by another program
            hscmd = re.search(r"hisat2 .+ -[1U] ([^\s,]+)", l)
            if hscmd:
                s_name = self.clean_s_name(hscmd.group(1), f['root'])
                log.debug("Found a HISAT2 command, updating sample name to '{}'".format(s_name))

            # Run through all regexes
            for k, r in regexes.items():
                match = re.search(r, l)
                if match:
                    parsed_data[k] = int(match.group(1))

            # Overall alignment rate
            overall = re.search(r"Overall alignment rate: ([\d\.]+)%", l)
            if overall:
                parsed_data['overall_alignment_rate'] = float(overall.group(1))

                # Save parsed data
                if s_name in self.hisat2_data:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
                self.add_data_source(f, s_name)
                self.hisat2_data[s_name] = parsed_data

                # Reset in case we find more in this log file
                s_name = f['s_name']
                parsed_data = {}


    def hisat2_general_stats_table(self):
        """ Take the parsed stats from the HISAT2 report and add it to the
        basic stats table at the top of the report """

        headers = OrderedDict()
        headers['overall_alignment_rate'] = {
            'title': '% Aligned',
            'description': 'overall alignment rate',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'YlGn'
        }
        self.general_stats_addcols(self.hisat2_data, headers)

    def hisat2_alignment_plot (self):
        """ Make the HighCharts HTML to plot the alignment rates """

        # Split the data into SE and PE
        sedata = {}
        pedata = {}
        for s_name, data in self.hisat2_data.items():
            if 'paired_total' in data:
                # Save half 'pairs' of mate counts
                m_keys = ['unpaired_total', 'unpaired_aligned_none', 'unpaired_aligned_one', 'unpaired_aligned_multi']
                for k in m_keys:
                    if k in data:
                        data[k] = float(data[k]) / 2.0
                pedata[s_name] = data
            else:
                sedata[s_name] = data

        # Two plots, don't mix SE with PE
        if len(sedata) > 0:
            sekeys = OrderedDict()
            sekeys['unpaired_aligned_one'] = { 'color': '#20568f', 'name': 'SE mapped uniquely' }
            sekeys['unpaired_aligned_multi'] = { 'color': '#f7a35c', 'name': 'SE multimapped' }
            sekeys['unpaired_aligned_none'] = { 'color': '#981919', 'name': 'SE not aligned' }
            pconfig = {
                'id': 'hisat2_se_plot',
                'title': 'HISAT2: SE Alignment Scores',
                'ylab': '# Reads',
                'cpswitch_counts_label': 'Number of Reads'
            }
            self.add_section(
                plot = bargraph.plot(sedata, sekeys, pconfig)
            )

        if len(pedata) > 0:
            pekeys = OrderedDict()
            pekeys['paired_aligned_one'] = { 'color': '#20568f', 'name': 'PE mapped uniquely' }
            pekeys['paired_aligned_discord_one'] = { 'color': '#5c94ca', 'name': 'PE mapped discordantly uniquely' }
            pekeys['unpaired_aligned_one'] = { 'color': '#95ceff', 'name': 'PE one mate mapped uniquely' }
            pekeys['paired_aligned_multi'] = { 'color': '#f7a35c', 'name': 'PE multimapped' }
            pekeys['unpaired_aligned_multi'] = { 'color': '#ffeb75', 'name': 'PE one mate multimapped' }
            pekeys['unpaired_aligned_none'] = { 'color': '#981919', 'name': 'PE neither mate aligned' }
            pconfig = {
                'id': 'hisat2_pe_plot',
                'title': 'HISAT2: PE Alignment Scores',
                'ylab': '# Reads',
                'cpswitch_counts_label': 'Number of Reads'
            }
            self.add_section(
                description = '<em>Please note that single mate alignment counts are halved to tally with pair counts properly.</em>',
                plot = bargraph.plot(pedata, pekeys, pconfig)
            )



#!/usr/bin/env python

""" MultiQC module to parse output from kraken """

from __future__ import print_function
from collections import OrderedDict
import os
import logging
import re

from multiqc import config
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """ Kraken module """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(
            name = 'Kraken',
            anchor = 'kraken',
            href = "https://ccb.jhu.edu/software/kraken/",
            info = "is a taxonomic classification tool that uses exact k-mer matches to find the lowest common ancestor (LCA) of a given sequence."
        )

        self.t_ranks = OrderedDict()
        self.t_ranks['S'] = 'Species'
        self.t_ranks['G'] = 'Genus'
        self.t_ranks['F'] = 'Family'
        self.t_ranks['O'] = 'Order'
        self.t_ranks['C'] = 'Class'
        self.t_ranks['P'] = 'Phylum'
        self.t_ranks['K'] = 'Kingdom'
        self.t_ranks['D'] = 'Domain'
        self.t_ranks['R'] = 'Root'
        # self.t_ranks['U'] = 'Unclassified'

        # Find and load any kraken reports
        self.kraken_raw_data = dict()
        for f in self.find_log_files('kraken', filehandles=True):
            self.parse_logs(f)

        # Filter to strip out ignored sample names
        self.kraken_raw_data = self.ignore_samples(self.kraken_raw_data)

        if len(self.kraken_raw_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.kraken_raw_data)))

        # Sum counts across all samples, so that we can pick top 5
        self.kraken_total_pct = dict()
        self.kraken_total_counts = dict()
        self.kraken_sample_total_readcounts = dict()
        self.sum_sample_counts()

        self.general_stats_cols()
        self.top_five_barplot()


    def parse_logs(self, f):
        """
        Parses a kraken report output file

        1. Percentage of fragments covered by the clade rooted at this taxon
        2. Number of fragments covered by the clade rooted at this taxon
        3. Number of fragments assigned directly to this taxon
        4. A rank code, indicating:
            * (U)nclassified
            * (R)oot
            * (D)omain
            * (K)ingdom
            * (P)hylum
            * (C)lass
            * (O)rder
            * (F)amily
            * (G)enus
            * (S)pecies
           Taxa that are not at any of these 10 ranks have a rank code that is
           formed by using the rank code of the closest ancestor rank with
           a number indicating the distance from that rank.  E.g., "G2" is a
           rank code indicating a taxon is between genus and species and the
           grandparent taxon is at the genus rank.
        5. NCBI taxonomic ID number
        6. Indented scientific name
        """

        # Search regexes for stats
        k2_regex = re.compile(r"^\s{1,2}(\d{1,2}\.\d{1,2})\t(\d+)\t(\d+)\t([\dUDKPCOFGS-]{1,3})\t(\d+)(\s+)(.+)")
        data = []
        for l in f['f']:
            match = k2_regex.search(l)
            if match:
                row = {
                    'percent': float(match.group(1)),
                    'counts_rooted': int(match.group(2)),
                    'counts_direct': int(match.group(3)),
                    'rank_code': match.group(4),
                    'tax_id': int(match.group(5)),
                    'num_spaces': len(match.group(6)),
                    'classif': match.group(7)
                }
                data.append(row)

        self.kraken_raw_data[f['s_name']] = data

    def sum_sample_counts(self):
        """ Sum counts across all samples for kraken data """

        # Sum the percentages for each taxa across all samples
        # Allows us to pick top-5 for each rank
        # Use percentages instead of counts so that deeply-sequences samples
        # are not unfairly over-represented
        for s_name, data in self.kraken_raw_data.items():
            total_guess_count = None
            for row in data:

                # Convenience vars that are easier to read
                rank_code = row['rank_code']
                classif = row['classif']

                # Skip anything that doesn't exactly fit a tax rank level
                if row['rank_code'] == '-' or any(c.isdigit() for c in row['rank_code']):
                    continue

                # Calculate the total read count using percentages
                # We use either unclassified or the first domain encountered, to try to use the largest proportion of reads = most accurate guess
                if rank_code == 'U' or (rank_code == 'D' and row['counts_rooted'] > total_guess_count):
                    self.kraken_sample_total_readcounts[s_name] = round(float(row['counts_rooted']) / (row['percent'] / 100.0))
                    total_guess_count = row['counts_rooted']

                if rank_code not in self.kraken_total_pct:
                    self.kraken_total_pct[rank_code] = dict()
                    self.kraken_total_counts[rank_code] = dict()

                if classif not in self.kraken_total_pct[rank_code]:
                    self.kraken_total_pct[rank_code][classif] = 0
                    self.kraken_total_counts[rank_code][classif] = 0
                self.kraken_total_pct[rank_code][classif] += row['percent']
                self.kraken_total_counts[rank_code][classif] += row['counts_rooted']

    def general_stats_cols(self):
        """ Add a couple of columns to the General Statistics table """

        # Get top taxa in most specific taxa rank that we have
        top_five = []
        top_rank_code = None
        top_rank_name = None
        for rank_code, rank_name in self.t_ranks.items():
            try:
                sorted_pct = sorted(self.kraken_total_pct[rank_code].items(), key=lambda x: x[1], reverse=True)
                for classif, pct_sum in sorted_pct[:5]:
                    top_five.append(classif)
                top_rank_code = rank_code
                top_rank_name = rank_name
                break
            except KeyError:
                # No species-level data found etc
                pass

        top_one_hkey = '% {}'.format(top_five[0])

        # Column headers
        headers = OrderedDict()
        headers[top_one_hkey] = {
            'title': top_one_hkey,
            'description': 'Percentage of reads that were the top {} over all samples - {}'.format(top_rank_name, top_five[0]),
            'suffix': '%',
            'max': 100,
            'scale': 'PuBuGn'
        }
        headers['% Top 5'] = {
            'title': '% Top 5 {}'.format(top_rank_name),
            'description': 'Percentage of reads that were classified by one of the top 5 {} ({})'.format(top_rank_name, ', '.join(top_five)),
            'suffix': '%',
            'max': 100,
            'scale': 'PuBu'
        }
        headers['% Unclassified'] = {
            'title': '% Unclassified',
            'description': 'Percentage of reads that were unclassified',
            'suffix': '%',
            'max': 100,
            'scale': 'OrRd'
        }

        # Get table data
        tdata = {}
        for s_name, d in self.kraken_raw_data.items():
            tdata[s_name] = {}
            for row in d:
                if row['rank_code'] == 'U':
                    tdata[s_name]['% Unclassified'] = row['percent']
                if row['rank_code'] == top_rank_code and row['classif'] in top_five:
                    tdata[s_name]['% Top 5'] = row['percent'] + tdata[s_name].get('% Top 5', 0)
                if row['rank_code'] == top_rank_code and row['classif'] == top_five[0]:
                    tdata[s_name][top_one_hkey] = row['percent']

            if top_one_hkey not in tdata[s_name]:
                tdata[s_name][top_one_hkey] = 0

        self.general_stats_addcols(tdata, headers)


    def top_five_barplot(self):
        """ Add a bar plot showing the top-5 from each taxa rank """

        pd = []
        cats = list()
        pconfig = {
            'id': 'kraken-topfive-plot',
            'title': 'Kraken 2: Top taxa',
            'ylab': 'Number of fragments',
            'data_labels': list(self.t_ranks.values())
        }

        for rank_code, rank_name in self.t_ranks.items():
            rank_cats = OrderedDict()
            rank_data = dict()

            # Loop through the summed tax percentages to get the top 5 across all samples
            try:
                sorted_pct = sorted(self.kraken_total_pct[rank_code].items(), key=lambda x: x[1], reverse=True)
            except KeyError:
                # Taxa rank not found in this sample
                continue
            i = 0
            counts_shown = {}
            for classif, pct_sum in sorted_pct:
                i += 1
                if i > 5:
                    break
                rank_cats[classif] = {'name': classif}
                # Pull out counts for this rank + classif from each sample
                for s_name, d in self.kraken_raw_data.items():
                    if s_name not in rank_data:
                        rank_data[s_name] = dict()
                    if s_name not in counts_shown:
                        counts_shown[s_name] = 0
                    for row in d:
                        if row['rank_code'] == rank_code:
                            if row['classif'] == classif:
                                if classif not in rank_data[s_name]:
                                    rank_data[s_name][classif] = 0
                                rank_data[s_name][classif] += row['counts_rooted']
                                counts_shown[s_name] += row['counts_rooted']

            # Add in unclassified reads and "other" - we presume from other species etc.
            for s_name, d in self.kraken_raw_data.items():
                for row in d:
                    if row['rank_code'] == 'U':
                        rank_data[s_name]['U'] = row['counts_rooted']
                        counts_shown[s_name] += row['counts_rooted']
                rank_data[s_name]['other'] = self.kraken_sample_total_readcounts[s_name] - counts_shown[s_name]

                # This should never happen... But it does sometimes if the total read count is a bit off
                if rank_data[s_name]['other'] < 0:
                    log.debug("Found negative 'other' count for {} ({}): {}".format(s_name, self.t_ranks[rank_code], rank_data[s_name]['other']))
                    rank_data[s_name]['other'] = 0

            rank_cats['other'] = { 'name': 'Other', 'color': '#cccccc' }
            rank_cats['U'] = { 'name': 'Unclassified', 'color': '#d4949c' }

            cats.append(rank_cats)
            pd.append(rank_data)

        self.add_section (
            name = 'Top taxa',
            anchor = 'kraken-topfive',
            description = 'The number of reads falling into the top 5 taxa across different ranks.',
            helptext = """
                To make this plot, the percentage of each sample assigned to a given taxa is summed across all samples.
                The counts for these top five taxa are then plotted for each of the 9 different taxa ranks.
                The unclassified count is always shown across all taxa ranks.

                The total number of reads is approximated by dividing the number of `unclassified` reads by the percentage of
                the library that they account for.
                Note that this is only an approximation, and that kraken percentages don't always add to exactly 100%.

                The category _"Other"_ shows the difference between the above total read count and the sum of the read counts
                in the top 5 taxa shown + unclassified. This should cover all taxa _not_ in the top 5, +/- any rounding errors.

                Note that any taxon that does not exactly fit a taxon rank (eg. `-` or `G2`) is ignored.
            """,
            plot = bargraph.plot(pd, cats, pconfig)
        )



#!/usr/bin/env python
""" MultiQC module to parse output from HiCExplorer """
from __future__ import print_function
from collections import OrderedDict
import logging

from multiqc import config
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

log = logging.getLogger(__name__)


class MultiqcModule(BaseMultiqcModule):
    def __init__(self):
        # Initialise the parent object
        super(MultiqcModule, self).__init__(
            name = 'HiCExplorer',
            anchor = 'hicexplorer',
            href = "https://hicexplorer.readthedocs.io",
            info = " addresses the common tasks of Hi-C analysis from processing to visualization."
        )

        self.hicexplorer_data = dict()
        for f in self.find_log_files('hicexplorer'):
            if f['s_name'] != "QC_table":
                # Parse the log file
                parsed_data = self.parse_logs(f['f'])
                # Build the sample ID
                s_name = "{}_{}_{}".format(f['root'], f['s_name'], parsed_data['File'])
                s_name = self.clean_s_name(s_name, f['root'])
                # Save results
                if s_name in self.hicexplorer_data:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
                self.hicexplorer_data[s_name] = parsed_data
                self.add_data_source(f, s_name=s_name)

        self.hicexplorer_data = self.ignore_samples(self.hicexplorer_data)

        if len(self.hicexplorer_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.hicexplorer_data)))

        self.write_data_file(self.hicexplorer_data, 'multiqc_hicexplorer')

        self.colors = [
            "#1f77b4",
            "#ff7f0e",
            "#2ca02c",
            "#d62728",
            "#9467bd",
            "#8c564b",
            "#7f7f7f",
            "#bcbd22",
            "#17becf",
            "#D2691E"
        ]

        # detect version of QC file
        # no 'Pairs mappable, unique and high quality' --> version 1.7
        # Contains 'pairs used' --> 1.8 - 3.1
        # Contains 'Hi-C contacts' --> since 3.2
        hicexplorer_versions = set()
        for s_name in self.hicexplorer_data:

            # compatibility to HiCExplorer <= 1.7 version QC files
            if not 'Pairs mappable, unique and high quality' in self.hicexplorer_data[s_name] and 'Pairs considered' in self.hicexplorer_data[s_name]:
                hicexplorer_versions.add('1.7')
                self.hicexplorer_data[s_name]['Pairs mappable, unique and high quality'] = self.hicexplorer_data[s_name]['Pairs considered']
                for key in ['One mate unmapped', 'One mate not unique', 'One mate low quality']:
                    self.hicexplorer_data[s_name]['Pairs mappable, unique and high quality'] -= self.hicexplorer_data[s_name][key]

            # compatibility to HiCExplorer <= 3.1 version QC files
            if 'Pairs considered' in self.hicexplorer_data[s_name]:
                self.hicexplorer_data[s_name]['Sequenced reads'] = self.hicexplorer_data[s_name]['Pairs considered']
                self.hicexplorer_data[s_name]['Hi-c contacts'] = self.hicexplorer_data[s_name]['Pairs used']
                self.hicexplorer_data[s_name]['Low mapping quality'] = self.hicexplorer_data[s_name]['One mate low quality']
                self.hicexplorer_data[s_name]['Intra short range (< 20kb)'] = self.hicexplorer_data[s_name]['Short range']
                self.hicexplorer_data[s_name]['Intra long range (>= 20kb)'] = self.hicexplorer_data[s_name]['Long range']
                self.hicexplorer_data[s_name]['Read pair type: inward pairs'] = self.hicexplorer_data[s_name]['Inward pairs']
                self.hicexplorer_data[s_name]['Read pair type: outward pairs'] = self.hicexplorer_data[s_name]['Outward pairs']
                self.hicexplorer_data[s_name]['Read pair type: left pairs'] = self.hicexplorer_data[s_name]['Left pairs']
                self.hicexplorer_data[s_name]['Read pair type: right pairs'] = self.hicexplorer_data[s_name]['Right pairs']

            elif 'Sequenced reads' in self.hicexplorer_data[s_name]:
                hicexplorer_versions.add('3.2')

        log.debug('HiCExplorer versions: {}'.format(', '.join(hicexplorer_versions)))

        # prepare the basic statistics for hicexplorer
        self.hicexplorer_basic_statistics()

        # key lists for plotting
        keys_categorization_of_reads_considered = [
            'Pairs mappable, unique and high quality',
            'One mate unmapped',
            'One mate not unique',
            'Low mapping quality'
        ]
        keys_mappable_unique_and_high_quality = [
            'Hi-c contacts',
            'Self ligation (removed)',
            'Same fragment',
            'Self circle',
            'Dangling end',
            'One mate not close to rest site',
            'Duplicated pairs'
        ]
        keys_list_contact_distance = [
            'Intra short range (< 20kb)',
            'Intra long range (>= 20kb)',
            'Inter chromosomal'
        ]
        keys_list_read_orientation = [
            'Read pair type: inward pairs',
            'Read pair type: outward pairs',
            'Read pair type: left pairs',
            'Read pair type: right pairs',
            'Inter chromosomal'
        ]

        # prepare the detail report section
        self.add_section(
            name='Mapping statistics',
            anchor='hicexplorer_categorization_of_considered_reads',
            plot=self.hicexplorer_create_plot(
                keys_categorization_of_reads_considered, 'HiCExplorer: Categorization of considered reads', 'categorization'),
            description='This shows how the sequenced read pairs were mapped and those filtered due to mapping problems.',
            helptext='''
                * **Pairs mappable, unique and high quality**
                    * The count of reads that were considered as valid reads and were not one of the following:
                * **One mate unmapped**
                    * Filtered out read because one mate was not mapped.
                * **One mate not unique**
                    * Filtered out read because one mate was not unique.
                * **Low mapping quality**
                    * Filtered out because one mate was having a low quality.
            '''
        )

        self.add_section(
            name='Read filtering',
            anchor='hicexplorer_pairs_categorized',
            plot=self.hicexplorer_create_plot(
                keys_mappable_unique_and_high_quality, 'HiCExplorer: Categorization of reads - Pairs mappable, unique and high quality', 'mapping'),
            description='This figure contains the number of reads that were finally used to build the '
            'Hi-C matrix along with the reads that where filtered out.',
            helptext="""
            * **Dangling ends**
                * These are reads that start with the restriction site and constitute reads that were digested but no ligated.
            * **Same fragment**
                * These are read mates, facing inward, separated by up to 800 bp that do not have a restriction enzyme in between.
                * These read pairs are not valid Hi-C pairs.
            * **Self circle**
                * Self circles are defined as pairs within 25kb with 'outward' read orientation
            * **Self ligation**
                * These are read pairs with a restriction site in between that are within 800 bp."""
        )

        self.add_section(
            name='Contact distance',
            anchor='hicexplorer_contact_distance',
            plot=self.hicexplorer_create_plot(
                keys_list_contact_distance, 'HiCExplorer: Contact distance', 'contact_distance'),
            description='This figure contains information about the distance and location of the valid pairs used.',
            helptext='''
            * **Intra long range**
                * Pairs with a distance greater than 20 kilobases
            * **Intra short range**
                * Pairs with a distance less than 20 kilobases
            * **Inter chromosomal**
                * Interchromosomal pairs.
            '''
        )

        self.add_section(
            name='Read orientation',
            anchor='hicexplorer_read_orientation',
            plot=self.hicexplorer_create_plot(
                keys_list_read_orientation, 'HiCExplorer: Read orientation', 'orientation'),
            description='This figure contains information about the orientation of the read pairs.',
            helptext='''
                * **Inward pairs**
                    * First mate is a forward read, second is reverse.
                    * `--------------->              <----------------`
                * **Outward pairs**
                    * First mate is a reverse read, second is forward.
                    * `--------------->              <----------------`
                * **Left pairs**
                    * Both are reverse reads.
                    * `<---------------              <----------------`
                * **Right pairs**
                    * Both are forward reads.
                    * `--------------->              ---------------->`
            '''
        )

    def parse_logs(self, f):
        """Parse a given HiCExplorer log file from hicBuildMatrix."""
        data = {}
        for l in f.splitlines():
            # catch empty lines
            if len(l) == 0:
                continue
            s = l.split("\t")
            # Skip header
            if s[0].startswith('#'):
                continue
            data_ = []
            # catch lines with descriptive content: "Of Hi-C contacts:"
            for i in s[1:]:
                if len(i) == 0:
                    continue
                try:
                    i = i.replace('(', '')
                    i = i.replace(')', '')
                    i = i.replace(',', '')
                    data_.append(float(i))
                except ValueError:
                    data_.append(i)
            if len(data_) == 0:
                continue
            if s[0].startswith('Intra short range (< 20kb)'):
                s[0] = 'Intra short range (< 20kb)'
            elif s[0].startswith('same fragment'):
                s[0] = 'same fragment'
            elif s[0].startswith('short range'):
                s[0] = 'short range'
            s[0] = s[0].capitalize()
            data[s[0]] = data_[0]
            try:
                data[s[0]+'_pct'] = data_[1]
            except IndexError:
                pass
        return data

    def hicexplorer_basic_statistics(self):
        """Create the general statistics for HiCExplorer."""
        data = {}
        for s_name in self.hicexplorer_data:
            max_distance_key = 'Max rest. site distance'
            total_pairs = self.hicexplorer_data[s_name]['Sequenced reads']
            try:
                self.hicexplorer_data[s_name][max_distance_key]
            except KeyError:
                max_distance_key = 'Max library insert size'
            data_ = {
                'Sequenced reads': self.hicexplorer_data[s_name]['Sequenced reads'],
                'Hi-c contacts': self.hicexplorer_data[s_name]['Hi-c contacts'] / total_pairs,
                'Mapped': self.hicexplorer_data[s_name]['One mate unmapped'] / total_pairs,
                'Min rest. site distance': self.hicexplorer_data[s_name]['Min rest. site distance'],
                max_distance_key: self.hicexplorer_data[s_name][max_distance_key],
            }
            data[s_name] = data_
        headers = OrderedDict()
        headers['Sequenced reads'] = {
            'title': '{} Pairs'.format(config.read_count_prefix),
            'description': 'Total number of read pairs ({})'.format(config.read_count_desc),
            'shared_key': 'read_count'
        }
        headers['Hi-c contacts'] = {
            'title': '% Used pairs',
            'max': 100,
            'min': 0,
            'modify': lambda x: x * 100,
            'suffix': '%'
        }
        headers['Mapped'] = {
            'title': '% Mapped',
            'max': 100,
            'min': 0,
            'modify': lambda x: (1 - x) * 100,
            'scale': 'RdYlGn',
            'suffix': '%'
        }
        headers['Min rest. site distance'] = {
            'title': 'Min RE dist',
            'description': 'Minimum restriction site distance (bp)',
            'format': '{:.0f}',
            'suffix': ' bp'
        }
        headers[max_distance_key] = {
            'title': 'Max RE dist',
            'description': '{} (bp)'.format(max_distance_key),
            'format': '{:.0f}',
            'suffix': ' bp'
        }

        self.general_stats_addcols(data, headers)

    def hicexplorer_create_plot(self, pKeyList, pTitle, pId):
        """Create the graphics containing information about the read quality."""

        keys = OrderedDict()
        for i, key_ in enumerate(pKeyList):
            keys[key_] = {'color': self.colors[i]}

        config = {
            'id': 'hicexplorer_' + pId + '_plot',
            'title': pTitle,
            'ylab': 'Number of Reads',
            'cpswitch_counts_label': 'Number of Reads'
        }

        return bargraph.plot(self.hicexplorer_data, keys, config)



#!/usr/bin/env python3

import logging
import re
from collections import OrderedDict
import traceback

from multiqc import config
from multiqc.modules.base_module import BaseMultiqcModule
from multiqc.plots import bargraph, linegraph

# Initialize log
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """FLASh MultiQC module

    Options:
      use_output_name: true - use first output filename as sample name
        default uses first input filename in log
    """
    def __init__(self):
        # Initialise the parent object
        super(MultiqcModule, self).__init__(
            name = 'FLASh',
            anchor = 'flash',
            href = "https://ccb.jhu.edu/software/FLASH/",
            info = "is a very fast and accurate software tool to merge paired-end reads from next-generation sequencing experiments."
        )

        # Find all log files with flash msgs
        self.flash_data = OrderedDict()
        for logfile in self.find_log_files('flash/log'):
            self.flash_data.update(self.parse_flash_log(logfile))

        # ignore sample names
        self.flash_data = self.ignore_samples(self.flash_data)

        try:
            if not self.flash_data:
                raise UserWarning
            log.info("Found %d log reports", len(self.flash_data))

            self.stats_table(self.flash_data)

            self.add_section(
                name='Read combination statistics',
                anchor='flash-bargraph',
                plot=self.summary_plot(self.flash_data))

            self.write_data_file(self.flash_data, 'multiqc_flash_combo_stats')

        except UserWarning:
            pass
        except Exception as err:
            log.error(err)
            log.debug(traceback.format_exc())

        ## parse histograms
        self.flash_hist = self.hist_results()

        # can't find any suitable logs
        if not self.flash_data and not self.flash_hist:
            raise UserWarning

    @staticmethod
    def split_log(logf):
        """split concat log into individual samples"""
        flashpatt = re.compile(
            r'\[FLASH\] Fast Length Adjustment of SHort reads\n(.+?)\[FLASH\] FLASH', flags=re.DOTALL)
        return flashpatt.findall(logf)

    @staticmethod
    def get_field(field, slog, fl=False):
        """parse sample log for field
           set fl=True to return a float
           otherwise, returns int
        """
        field += r'\:\s+([\d\.]+)'
        match = re.search(field, slog)
        if match:
            if fl:
                return float(match.group(1))
            return int(match.group(1))
        return 0

    def clean_pe_name(self, nlog, root):
        """additional name cleaning for paired end data"""
        use_output_name = getattr(config, 'flash', {}).get('use_output_name', False)
        if use_output_name:
            name = re.search(r'Output files\:\n\[FLASH\]\s+(.+?)\n', nlog)
        else:
            name = re.search(r'Input files\:\n\[FLASH\]\s+(.+?)\n', nlog)
        if not name:
            return None
        name = name.group(1)
        name = self.clean_s_name(name, root)
        return name

    def parse_flash_log(self, logf):
        """parse flash logs"""
        data = OrderedDict()
        samplelogs = self.split_log(logf['f'])
        for slog in samplelogs:
            try:
                sample = dict()
                ## Sample name ##
                s_name = self.clean_pe_name(slog, logf['root'])
                if s_name is None:
                    continue
                sample['s_name'] = s_name

                ## Log attributes ##
                sample['totalpairs'] = self.get_field('Total pairs', slog)
                sample['discardpairs'] = self.get_field('Discarded pairs', slog)
                sample['percdiscard'] = self.get_field('Percent Discarded', slog, fl=True)
                sample['combopairs'] = self.get_field('Combined pairs', slog)
                sample['inniepairs'] = self.get_field('Innie pairs', slog)
                sample['outiepairs'] = self.get_field('Outie pairs', slog)
                sample['uncombopairs'] = self.get_field('Uncombined pairs', slog)
                sample['perccombo'] = self.get_field('Percent combined', slog, fl=True)

                data[s_name] = sample
            except Exception as err:
                log.warning("Error parsing record in {}. {}".format(logf['fn'], err))
                log.debug(traceback.format_exc())
                continue
        return data

    def stats_table(self, data):
        """Add percent combined to general stats table"""
        headers = OrderedDict()
        headers['combopairs'] = {
            'title': 'Combined pairs',
            'description': 'Num read pairs combined',
            'shared_key': 'read_count',
            'hidden': True,
            'scale': False
        }
        headers['perccombo'] = {
            'title': '% Combined',
            'description': '% read pairs combined',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'PiYG'
        }
        self.general_stats_addcols(data, headers)

    @staticmethod
    def summary_plot(data):
        """Barplot of combined pairs"""
        cats = OrderedDict()
        cats = {
            'combopairs': {
                'name': 'Combined pairs',
                'color': '#191970'
            },
            'inniepairs': {
                'name': 'Combined innie pairs',
                'color': '#191970'
            },
            'outiepairs': {
                'name': 'Combined outie pairs',
                'color': '#00A08A'
            },
            'uncombopairs': {
                'name': 'Uncombined pairs',
                'color': '#cd1076'
            },
            'discardpairs': {
                'name': 'Discarded pairs',
                'color': '#ffd700'
            }
        }
        splotconfig = {
            'id': 'flash_combo_stats_plot',
            'title': 'FLASh: Read combination statistics',
            'ylab': 'Number of read pairs'
        }
        # Only plot the combopairs category if we don't have inniepairs and outiepairs
        for s_name, d in data.items():
            if (d['inniepairs'] > 0 or d['outiepairs'] > 0) and d['combopairs'] > 0:
                del data[s_name]['combopairs']

        return bargraph.plot(data, cats, splotconfig)

    @staticmethod
    def parse_hist_files(histf):
        """parse histogram files"""
        nameddata = dict()
        data = dict()
        try:
            for l in histf['f'].splitlines():
                s = l.split()
                if s:
                    if len(s) != 2:
                        raise RuntimeError("invalid format: " + str(len(s)) + " column(s) found in row. must be exactly 2.")
                    data[int(s[0])] = int(s[1])
        except Exception as err:
            log.warning("Error parsing %s. %s", histf['fn'], err)
            log.debug(traceback.format_exc())
        else:
            if data:
                nameddata[histf['s_name']] = data
            else:
                log.debug("%s is empty.", histf['fn'])
        finally:
            return nameddata

    @staticmethod
    def get_colors(n):
        """get colors for freqpoly graph"""
        cb_palette = ["#E69F00", "#56B4E9", "#009E73", "#F0E442", "#0072B2", "#D55E00",
                      "#CC79A7","#001F3F", "#0074D9", "#7FDBFF", "#39CCCC", "#3D9970", "#2ECC40",
                       "#01FF70", "#FFDC00", "#FF851B", "#FF4136", "#F012BE", "#B10DC9",
                       "#85144B", "#AAAAAA", "#000000"]

        whole = int(n/22)
        extra = (n % 22)
        cols = cb_palette * whole
        if extra >= 0:
            cols.extend(cb_palette[0:extra])
        return cols

    @staticmethod
    def freqpoly_plot(data):
        """make freqpoly plot of merged read lengths"""
        rel_data = OrderedDict()
        for key, val in data.items():
            tot = sum(val.values(), 0)
            rel_data[key] = {k: v / tot for k, v in val.items()}
        fplotconfig = {
            'id': 'flash_freqpoly_plot',
            'title': 'FLASh: Frequency of merged read lengths',
            'xlab': 'Merged Read Length',
            'ylab': 'Frequency',
            'data_labels': [
                {'name': 'Absolute', 'ylab': 'Frequency', 'xlab': 'Merged Read Length'},
                {'name': 'Relative', 'ylab': 'Relative Frequency', 'xlab': 'Merged Read Length'}
            ],
            'colors': dict(zip(data.keys(), MultiqcModule.get_colors(len(data))))
        }
        return linegraph.plot([data, rel_data], fplotconfig)

    def hist_results(self):
        """process flash numeric histograms"""
        self.hist_data = OrderedDict()
        for histfile in self.find_log_files('flash/hist'):
            self.hist_data.update(self.parse_hist_files(histfile))

        # ignore sample names
        self.hist_data = self.ignore_samples(self.hist_data)

        try:
            if not self.hist_data:
                raise UserWarning
            log.info("Found %d histogram reports", len(self.hist_data))

            self.add_section(
                name = 'Frequency polygons of merged read lengths',
                anchor = 'flash-histogram',
                description = 'This plot is made from the numerical histograms output by FLASh.',
                plot = self.freqpoly_plot(self.hist_data)
            )

        except UserWarning:
            pass
        except Exception as err:
            log.error(err)
            log.debug(traceback.format_exc())
        return len(self.hist_data)



#!/usr/bin/env python

""" MultiQC module to parse output from HOMER findpeaks """

import logging
import os
from collections import OrderedDict

# Initialise the logger
log = logging.getLogger(__name__)


class FindPeaksReportMixin():

    def parse_homer_findpeaks(self):
        """ Find HOMER findpeaks logs and parse their data """

        self.homer_findpeaks = dict()
        for f in self.find_log_files('homer/findpeaks', filehandles=True):
            self.parse_findPeaks(f)

        # Filter to strip out ignored sample names
        self.homer_findpeaks = self.ignore_samples(self.homer_findpeaks)

        if len(self.homer_findpeaks) > 0:

            # Write parsed report data to a file
            self.write_data_file(self.homer_findpeaks, 'multiqc_homer_findpeaks')

            # General Stats Table
            stats_headers = OrderedDict()
            stats_headers['approximate_ip_efficiency'] = {
                'title': '% Efficiency',
                'description': 'Approximate IP efficiency',
                'min': 0,
                'max': 100,
                'suffix': '%',
                'scale': 'RdYlGn'
            }
            stats_headers['total_peaks'] = {
                'title': 'Total Peaks',
                'min': 0,
                'format': '{:,.0f}',
                'scale': 'GnBu'
            }
            stats_headers['expected_tags_per_peak'] = {
                'title': 'Tags/Peak',
                'description': 'Expected tags per peak',
                'min': 0,
                'format': '{:,.0f}',
                'scale': 'PuRd'
            }
            self.general_stats_addcols(self.homer_findpeaks, stats_headers, 'HOMER findpeaks')

        return len(self.homer_findpeaks)

    def parse_findPeaks(self, f):
        """ Parse HOMER findPeaks file headers. """
        parsed_data = dict()
        s_name = f['s_name']
        for l in f['f']:
            # Start of data
            if l.strip() and not l.strip().startswith('#'):
                break
            # Automatically parse header lines by = symbol
            s = l[2:].split('=')
            if len(s) > 1:
                k = s[0].strip().replace(' ','_').lower()
                v = s[1].strip().replace('%','')
                try:
                    parsed_data[k] = float(v)
                except ValueError:
                    parsed_data[k] = v
                if k == 'tag_directory':
                    s_name = self.clean_s_name(os.path.basename(v), os.path.dirname(v))

        if len(parsed_data) > 0:
            if s_name in self.homer_findpeaks:
                log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f['fn'], s_name))
            self.add_data_source(f, s_name, section='findPeaks')
            self.homer_findpeaks[s_name] = parsed_data



#!/usr/bin/env python

""" MultiQC submodule to parse output from RSeQC read_GC.py
http://rseqc.sourceforge.net/#read-gc-py """

from collections import OrderedDict
import logging

from multiqc.plots import linegraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """ Find RSeQC read_GC reports and parse their data """

    # Set up vars
    self.read_gc = dict()
    self.read_gc_pct = dict()

    # Go through files and parse data
    for f in self.find_log_files('rseqc/read_gc'):

        if f['f'].startswith('GC%	read_count'):
            gc = list()
            counts = list()
            for l in f['f'].splitlines():
                s = l.split()
                try:
                    gc.append(float(s[0]))
                    counts.append(float(s[1]))
                except:
                    pass
            if len(gc) > 0:
                sorted_gc_keys = sorted(range(len(gc)), key=lambda k: gc[k])
                total = sum(counts)
                if f['s_name'] in self.read_gc:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
                self.add_data_source(f, section='read_GC')
                self.read_gc[f['s_name']] = OrderedDict()
                self.read_gc_pct[f['s_name']] = OrderedDict()
                for i in sorted_gc_keys:
                    self.read_gc[f['s_name']][gc[i]] = counts[i]
                    self.read_gc_pct[f['s_name']][gc[i]] = (counts[i]/total)*100

    # Filter to strip out ignored sample names
    self.read_gc = self.ignore_samples(self.read_gc)

    if len(self.read_gc) > 0:

        # Add line graph to section
        pconfig = {
            'id': 'rseqc_read_gc_plot',
            'title': 'RSeQC: Read GC Content',
            'ylab': 'Number of Reads',
            'xlab': "GC content (%)",
            'xmin': 0,
            'xmax': 100,
            'tt_label': "<strong>{point.x}% GC</strong>: {point.y:.2f}",
            'data_labels': [
                {'name': 'Counts', 'ylab': 'Number of Reads'},
                {'name': 'Percentages', 'ylab': 'Percentage of Reads'}
            ]
        }
        self.add_section (
            name = 'Read GC Content',
            anchor = 'rseqc-read_gc',
            description = '<a href="http://rseqc.sourceforge.net/#read-gc-py" target="_blank">read_GC</a>' \
                " calculates a histogram of read GC content.</p>",
            plot = linegraph.plot([self.read_gc, self.read_gc_pct], pconfig)
        )

    # Return number of samples found
    return len(self.read_gc)



from itertools import chain

from multiqc.plots import linegraph

def plot_ihist(samples, file_type, **plot_args):
    """ Create line graph plot for basic histogram data for 'ihist'.

    The 'samples' parameter could be from the bbmap mod_data dictionary:
    samples = bbmap.MultiqcModule.mod_data[file_type]
    """

    sumy = sum([int(samples[sample]['data'][x][0])
                for sample in samples
                for x in samples[sample]['data']])

    cutoff = sumy * 0.99
    all_x = set()
    for item in sorted(chain(*[samples[sample]['data'].items()
                                for sample in samples])):
        all_x.add(item[0])
        cutoff -= item[1][0]
        if cutoff < 0:
            xmax = item[0]
            break
    else:
        xmax = max(all_x)

    data = {
        sample: {
            x: samples[sample]['data'][x][0] if x in samples[sample]['data'] else 0
            for x in all_x
        }
        for sample in samples
    }

    plot_params = {
            'id': 'bbmap-' + file_type + '_plot',
            'title': 'BBTools: ' + plot_args['plot_title'],
            'xmax': xmax,
            'xlab': 'Insert size (base pairs)',
            'ylab': 'Read count'
    }
    plot_params.update(plot_args['plot_params'])
    plot = linegraph.plot(
        data,
        plot_params
    )

    return plot



#!/usr/bin/env python

''' MultiQC module to parse output from BISCUITqc '''

from __future__ import print_function
from collections import OrderedDict
import logging
import re

from multiqc import config
from multiqc.plots import linegraph, bargraph, table, beeswarm
from multiqc.modules.base_module import BaseMultiqcModule

# Initialize the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    '''
    Inherits from base module class
    Initialize data structures and prepare BISCUIT report
    Inputs:
        No inputs
    Returns:
        BISCUIT report for MultiQC
    '''
    def __init__(self):
        # Initialize the parent object
        super(MultiqcModule, self).__init__(
            name = 'BISCUIT',
            anchor = 'biscuit',
            href = 'https://github.com/huishenlab/biscuit',
            info = '''
                is a tool to map bisulfite converted DNA sequence reads and
                determine cytosine methylation states.
            '''
        )

        # Set up data structures
        self.mdata = {
            # General statistics
            'align_mapq': {},
            'align_strand': {},
            'align_isize': {},
            # Duplicate reporting
            'dup_report': {},
            # Uniformity
            'qc_cv': {},
            # Base coverage
            'covdist_all_base_botgc': {},
            'covdist_all_base': {},
            'covdist_all_base_topgc': {},
            'covdist_q40_base_botgc': {},
            'covdist_q40_base': {},
            'covdist_q40_base_topgc': {},
            # CpG coverage
            'covdist_all_cpg_botgc': {},
            'covdist_all_cpg': {},
            'covdist_all_cpg_topgc': {},
            'covdist_q40_cpg_botgc': {},
            'covdist_q40_cpg': {},
            'covdist_q40_cpg_topgc': {},
            # Cytosine retention
            'cpg_retention_readpos': {},
            'cph_retention_readpos': {},
            'base_avg_retention_rate': {},
            'read_avg_retention_rate': {}
        }

        # NB: Cleaning filenames like this means that some MultiQC functionality, like -s / --fullnames doesn't work.
        # However, because some of the parsing relies on cleaned filenames, the above options break the
        # module if we use the centralised MultiQC functions.
        file_suffixes = [
            # General statistics
            '.txt',
            '_mapq_table',
            '_strand_table',
            '_isize_table',
            # Duplicate reporting
            '_dup_report',
            # Uniformity
            '_cv_table',
            # Base coverage
            '_covdist_all_base_botgc_table',
            '_covdist_all_base_table',
            '_covdist_all_base_topgc_table',
            '_covdist_q40_base_botgc_table',
            '_covdist_q40_base_table',
            '_covdist_q40_base_topgc_table',
            # CpG coverage
            '_covdist_all_cpg_botgc_table',
            '_covdist_all_cpg_table',
            '_covdist_all_cpg_topgc_table',
            '_covdist_q40_cpg_botgc_table',
            '_covdist_q40_cpg_table',
            '_covdist_q40_cpg_topgc_table',
            # Cytosine retention
            '_CpGRetentionByReadPos',
            '_CpHRetentionByReadPos',
            '_totalBaseConversionRate',
            '_totalReadConversionRate'
        ]

        # Find and parse alignment reports
        for k in self.mdata:
            for f in self.find_log_files('biscuit/{}'.format(k)):

                s_name = f['fn']
                for suffix in file_suffixes:
                    s_name = s_name.replace(suffix, '')
                s_name = self.clean_s_name(s_name, f['root'])

                # Add source file to multiqc_sources.txt
                self.add_data_source(f, s_name=s_name, section=k)

                if s_name in self.mdata[k]:
                    log.debug('Duplicate sample name found in {}! Overwriting: {}'.format(f['fn'], s_name))

                self.mdata[k][s_name] = getattr(self, 'parse_logs_{}'.format(k))(f['f'], f['fn'])

        for k in self.mdata:
            self.mdata[k] = self.ignore_samples(self.mdata[k])

        n_samples = max([len(self.mdata[k]) for k in self.mdata])

        if n_samples == 0:
            raise UserWarning

        log.info('Found {} samples'.format(n_samples))

        # Basic stats table
        self.biscuit_stats_table()

        # Make report sections
        for k in self.mdata:
            if len(self.mdata[k]) > 0:
                log.debug('Found {} {} reports'.format(len(self.mdata[k]), k))
                getattr(self, 'chart_{}'.format(k))()

    def biscuit_stats_table(self):
        '''
        Create general statistics table for BISCUIT data
        Inputs:
            Uses mdata['align_mapq'] and mdata['dup_report']
        Returns:
            Add columns to MultiQC general statistics table
        '''
        pd = {}

        # Calculate % aligned
        for s_name, dd in self.mdata['align_mapq'].items():
            if len(dd) > 0:
                total = sum([int(_) for _ in dd.values()])
                pd[s_name] = {'aligned': 100.0 * float(total - int(dd['unmapped'])) / total}

        # Caclulate % duplicated
        for s_name, dd in self.mdata['dup_report'].items():
            if s_name not in pd:
                pd[s_name] = {}
            if 'all' in dd and dd['all'] != -1:
                pd[s_name]['dup_all'] = dd['all']
            if 'q40' in dd and dd['q40'] != -1:
                pd[s_name]['dup_q40'] = dd['q40']

        pheader = OrderedDict()
        pheader['dup_q40'] = {
            'title': 'Dup. % for Q40 Reads',
            'min': 0,
            'max': 100,
            'suffix': '%',
            'scale': 'YlOrBr',
            'hidden': True
        }
        pheader['dup_all'] = {
            'title': 'Dup. % for All Reads',
            'min': 0,
            'max': 100,
            'suffix': '%',
            'scale': 'Reds'
        }
        pheader['aligned'] = {
            'title': '% Aligned',
            'min': 0,
            'max': 100,
            'suffix': '%',
            'scale': 'RdYlGn'
        }

        self.general_stats_addcols(pd, pheader)

    ########################################
    #####  General Mapping Information #####
    ########################################
    def parse_logs_align_mapq(self, f, fn):
        '''
        Parse _mapq_table.txt
        Inputs:
            f - current matched file
            fn - filename
        Returns:
            data - dictionary of aligned mapq data
        '''
        file_data = f.splitlines()[2:]

        # Handle missing data
        if len(file_data) == 0:
            log.debug('No data available in {}. Will not fill corresponding entries.'.format(fn))
            return {}

        data = {}
        for l in file_data:
            s = l.split()
            data[s[0]] = s[1] # data[MAPQ] = number of reads

        return data


    ########################################
    ####    Alignment Quality Report    ####
    ########################################
    def chart_align_mapq(self):
        '''
        Chart _mapq_table.txt
        Inputs:
            No inputs
        Returns:
            No returns, generates Mapping Overview and Mapping Quality
            Distribution charts
        '''

        #
        # Mapping Overview bar chart
        #
        pd = {}

        # Calculate alignment counts
        for s_name, dd in self.mdata['align_mapq'].items():
            if len(dd) > 0:
                pd[s_name] = {'opt_align': 0, 'sub_align': 0, 'not_align': 0}
                for mapq, cnt in dd.items():
                    if mapq == 'unmapped':
                        pd[s_name]['not_align'] += int(cnt)
                    elif int(mapq) >= 40:
                        pd[s_name]['opt_align'] += int(cnt)
                    else:
                        pd[s_name]['sub_align'] += int(cnt)

        pheader = OrderedDict()
        pheader['opt_align'] = {'color': '#1f78b4', 'name': 'Optimally Aligned Reads'}
        pheader['sub_align'] = {'color': '#a6cee3', 'name': 'Suboptimally Aligned Reads'}
        pheader['not_align'] = {'color': '#b2df8a', 'name': 'Unaligned Reads'}

        pconfig = {
            'id': 'biscuit-mapping-overview-plot',
            'title': 'BISCUIT: Mapping Overview',
            'ylab': 'Number of Reads',
            'cpswitch_counts_label': '# Reads'
        }

        self.add_section(
            name = 'Mapping Overview',
            anchor = 'biscuit-mapping-overview',
            description = '''
                Number of optimally aligned reads (`MAPQ>=40`), suboptimally
                aligned reads (`MAPQ<40`), and unmapped reads. Primary alignments only.
            ''',
            helptext = '''
                A good library should have a high fraction of reads
                that are optimally aligned. Note, suboptimally aligned reads
                include both non-unique alignments and imperfect alignments.
            ''',
            plot = bargraph.plot(pd, pheader, pconfig)
        )

        #
        # Mapping Quality Distribution line graph
        #

        # Calculate the % aligned for each mapping q score
        pd_mapq = {}
        for s_name, dd in self.mdata['align_mapq'].items():
            if len(dd) > 0:
                # Total aligned count across all qualities
                total = sum([int(cnt) for _, cnt in dd.items() if _ != 'unmapped'])
                cnts = []
                for mapq in range(61):
                    if str(mapq) in dd:
                        cnts.append(100.0 * float(dd[str(mapq)]) / total)
                    else:
                        cnts.append(0)
                pd_mapq[s_name] = dict(zip(range(61), cnts))

        pconfig = {
            'id': 'biscuit_mapq',
            'title': 'BISCUIT: Distribution of Mapping Qualities',
            'ymin': 0,
            'xmin': 0,
            'yLabelFormat': '{value}%',
            'tt_label': '<strong>Q{point.x}:</strong> {point.y:.2f}% of mapped reads',
            'ylab': '% of Primary Mapped Reads',
            'xlab': 'Mapping Quality Score'
        }

        self.add_section(
            name = 'Mapping Quality Distribution',
            anchor = 'biscuit-mapq',
            description = '''
                The percentage of the total number of mapped reads
                for each mapping quality score. Primary alignments only.
            ''',
            helptext = '''
                A good quality sample should have a high quality mapping score
                for the majority of alignments.
            ''',
            plot = linegraph.plot(pd_mapq, pconfig)
        )

    ########################################
    ####     Strand Alignment Report    ####
    ########################################
    def parse_logs_align_strand(self, f, fn):
        '''
        Parse _strand_table.txt
        Inputs:
            f - current matched file
            fn - filename
        Returns:
            data - dictionary of strand data for reads 1 and 2
        '''
        patterns = [
            r'(R1)\s+\((f)\)\:\s+(\d+)\s+(\d+)',
            r'(R1)\s+\((r)\)\:\s+(\d+)\s+(\d+)',
            r'(R2)\s+\((f)\)\:\s+(\d+)\s+(\d+)',
            r'(R2)\s+\((r)\)\:\s+(\d+)\s+(\d+)'
        ]

        data = {'read1': {}, 'read2': {}}
        for pat in patterns:
            m = re.search(pat, f, re.MULTILINE)
            if m is not None:
                if m.group(1) == 'R1':
                    if m.group(2) == 'f':
                        data['read1']['ff'] = int(m.group(3))
                        data['read1']['fr'] = int(m.group(4))
                    else:
                        data['read1']['rf'] = int(m.group(3))
                        data['read1']['rr'] = int(m.group(4))
                else:
                    if m.group(2) == 'f':
                        data['read2']['ff'] = int(m.group(3))
                        data['read2']['fr'] = int(m.group(4))
                    else:
                        data['read2']['rf'] = int(m.group(3))
                        data['read2']['rr'] = int(m.group(4))

        return data

    def chart_align_strand(self):
        '''
        Chart _strand_table.txt
        Inputs:
            No inputs
        Returns:
            No returns, generates Mapping Strand Distribution chart
        '''

        pd1 = {}
        pd2 = {}
        for s_name, dd in self.mdata['align_strand'].items():
            if len(dd['read1']) > 0:
                pd1[s_name] = dd['read1']
            if len(dd['read2']) > 0:
                pd2[s_name] = dd['read2']

        pheader = OrderedDict([
            ('ff', {'color': '#F53855', 'name': 'ff: Watson-Aligned, Watson-Bisulfite Conversion'}),
            ('fr', {'color': '#E37B40', 'name': 'fr: Watson-Aligned, Crick-Bisulfite Conversion' }),
            ('rf', {'color': '#46B29D', 'name': 'rf: Crick-Aligned, Watson-Bisulfite Conversion' }),
            ('rr', {'color': '#324D5C', 'name': 'rr: Crick-Aligned, Crick-Bisulfite Conversion'  })
        ])

        pconfig = {
            'id': 'biscuit_strands',
            'title': 'BISCUIT: Mapping Strand Distribution',
            'ylab': 'Number of Reads',
            'cpswitch_counts_label': '# Reads',
            'data_labels': [
                {'name': 'Read 1'},
                {'name': 'Read 2'}
            ]
        }

        # TODO: When PBAT mode is implemented, add comment in help text about
        #       how to interpret PBAT mode results
        if max(len(pd1), len(pd2)) > 0:
            self.add_section(
                name = 'Mapping Strand Distribution',
                anchor = 'biscuit-strands',
                description = 'For primary alignments, shows the number of reads mapped to each strand.',
                helptext = '''
                    Most bisulfite libraries typically map Read 1 to the parent
                    strand (`ff`, `rr`) and Read 2 to the daughter / synthesized
                    strand (`fr`, `rf`).

                    Note that PBAT and many single-cell / low input
                    libraries may not follow this assumption.
                ''',
                plot = bargraph.plot([pd1, pd2], [pheader, pheader], pconfig)
            )


    ########################################
    ####       Insert Size Report       ####
    ########################################
    def parse_logs_align_isize(self, f, fn):
        '''
        Parse _isize_table.txt
        Inputs:
            f - current matched file
            fn - filename
        Returns:
            data - dictionary of insert size data
        '''
        file_data = f.splitlines()[2:]

        # Handle missing data
        if len(file_data) == 0:
            log.debug('No data available in {}. Will not fill corresponding entries.'.format(fn))
            return {'no_data_available': 1}

        data = {'percent': {}, 'readcnt': {}}
        for l in file_data:
            fields = l.split('\t')
            data['percent'][int(fields[0])] = 100.0 * float(fields[1])
            data['readcnt'][int(fields[0])] = float(fields[2])

        return data

    def chart_align_isize(self):
        '''
        Chart _isize_table.txt
        Inputs:
            No inputs
        Returns:
            No returns, generates Insert Size Distribution chart
        '''

        pd_p = {}
        pd_r = {}
        for s_name, dd in self.mdata['align_isize'].items():
            if 'no_data_available' not in dd.keys():
                pd_p[s_name] = dd['percent']
                pd_r[s_name] = { ins: cnt * config.read_count_multiplier for ins, cnt in dd['readcnt'].items() }

        pconfig = {
            'id': 'biscuit_isize',
            'title': 'BISCUIT: Insert Size Distribution',
            'ymin': 0,
            'xmin': 0,
            'yLabelFormat': '{value}',
            'smooth_points': 1000, # limit number of points / smooth data
            'tt_label': '<strong>IS{point.x}:</strong> {point.y:.2f}',
            'xlab': 'Insert Size',
            'ylab': '% of Mapped Reads',
            'data_labels': [
                {
                    'name': 'Percent of Reads',
                    'ylab':'% of Mapped Reads'
                },
                {
                    'name': '{} of Reads'.format(config.read_count_desc.capitalize()),
                    'ylab':'{} of Mapped Reads'.format(config.read_count_desc.capitalize())
                }
            ]
        }

        self.add_section(
            name = 'Insert Size Distribution',
            anchor = 'biscuit-isize',
            description = 'Shows the distribution of insert sizes.',
            helptext = '''
                Insert size is defined as:

                ```
                (right-most coordinate of reverse-mate read) - (left-most coordinate of forward-mate read)
                ```

                Insert sizes are calculated for reads with a _"mapped in
                proper pair"_ `samtools` flag, and `MAPQ >= 40`.
            ''',
            plot = linegraph.plot([pd_p, pd_r], pconfig)
        )

    ########################################
    ####        Duplicate Report        ####
    ########################################
    def parse_logs_dup_report(self, f, fn):
        '''
        Parses _dup_report.txt
        Inputs:
            f - current matched file
            fn - filename
        Returns:
            data - dictionary of duplicate fractions
        '''
        patterns = [
            (
                r'Number of duplicate reads:\s+(\d+)',
                r'Number of reads:\s+(\d+)',
                'all'
            ),
            (
                r'Number of duplicate q40-reads:\s+(\d+)',
                r'Number of q40-reads:\s+(\d+)',
                'q40'
            )
        ]

        data = {}
        for pat_dup, pat_tot, key in patterns:
            m1 = re.search(pat_dup, f, re.MULTILINE)
            m2 = re.search(pat_tot, f, re.MULTILINE)
            if m1 is not None and m2 is not None:
                data[key] = (100.0 * float(m1.group(1)) / float(m2.group(1)))
            else:
                data[key] = -1

        return data

    def chart_dup_report(self):
        '''
        Charts _dup_report.txt
        Inputs:
            No inputs
        Returns:
            No returns, generates Duplicate Rates chart
        '''

        pd = dict([(s_name, dd) for s_name, dd in self.mdata['dup_report'].items() if dd['all'] != -1])

        pheader = OrderedDict()
        pheader['all'] = {
            'title': 'Overall',
            'suffix': '%',
            'max': 100,
            'min': 0,
            'scale': 'Reds'
        }
        pheader['q40'] = {
            'title': 'MAPQ >= 40',
            'suffix': '%',
            'max': 100,
            'min': 0,
            'scale': 'Purples'
        }

        pconfig = {
            'id': 'biscuit_dup_report',
            'table_title': 'BISCUIT: Duplicate Rates',
            'sortRows': False,
            'save_file': True
        }

        if len(pd) > 0:
            self.add_section(
                name = 'Duplicate Rates',
                anchor = 'biscuit-dup-report',
                description = 'Shows the percentage of total reads that are duplicates.',
                helptext = '''
                    `MAPQ >= 40` shows the duplicate rate for just the reads reads
                    with an alignment quality score of `MAPQ >= 40`.
                ''',
                plot = table.plot(pd, pheader, pconfig)
            )

    ########################################
    ####      Depths and Uniformity     ####
    ########################################
    def parse_logs_qc_cv(self, f, fn):
        '''
        Parses _cv_table.txt
        Inputs:
            f - current matched file
            fn - filename
        Returns:
            data - dictionary of depth uniformity measures
        '''

        data = {}
        targets = [
            'all_base',
            'all_cpg',
            'q40_base',
            'q40_cpg',
            'all_base_botgc',
            'all_cpg_botgc',
            'q40_base_botgc',
            'q40_cpg_botgc',
            'all_base_topgc',
            'all_cpg_topgc',
            'q40_base_topgc',
            'q40_cpg_topgc'
        ]
        for t in targets:
            m = re.search(
                '{}\t([\d\.]+)\t([\d\.]+)\t([\d\.]+)'.format(t),
                f, re.MULTILINE
            )
            if m is not None:
                data[t] = {
                    'mu': float(m.group(1)),
                    'sigma': float(m.group(2)),
                    'cv': float(m.group(3))
                }
            else:
                data[t] = {'mu': -1, 'sigma': -1, 'cv': -1}

        return data

    def chart_qc_cv(self):
        '''
        Charts _cv_table.txt
        Inputs:
            No inputs
        Returns:
            No returns, generates Sequencing Depth - Whole Genome chart
        '''

        cats = [
            ('all_base', 'a_b'),
            ('q40_base', 'q_b'),
            ('all_base_botgc', 'a_b_b'),
            ('q40_base_botgc', 'q_b_b'),
            ('all_base_topgc', 'a_b_t'),
            ('q40_base_topgc', 'q_b_t'),
            ('all_cpg', 'a_c'),
            ('q40_cpg', 'q_c'),
            ('all_cpg_botgc', 'a_c_b'),
            ('q40_cpg_botgc', 'q_c_b'),
            ('all_cpg_topgc', 'a_c_t'),
            ('q40_cpg_topgc', 'q_c_t')
        ]

        pd = OrderedDict()
        for s_name, dd in self.mdata['qc_cv'].items():
            data = OrderedDict()
            for cat, key in cats:
                if cat in dd:
                    if dd[cat]['mu'] != -1:
                        data['mu_'+key] = dd[cat]['mu']
                        data['cv_'+key] = dd[cat]['cv']
            if len(data) > 0:
                pd[s_name] = data

        shared_mean = {'min': 0, 'format': '{:,3f}', 'minRange': 10}
        shared_cofv = {'min': 0, 'format': '{:,3f}', 'minRange': 50}

        pheader = OrderedDict()
        pheader['mu_a_b'] = dict(shared_mean, **{'title': 'All Genome Mean', 'description': 'Mean Sequencing Depth for All Reads'})
        pheader['mu_q_b'] = dict(shared_mean, **{'title': 'Q40 Genome Mean', 'description': 'Mean Sequencing Depth for Q40 Reads'})
        pheader['mu_a_b_b'] = dict(shared_mean, **{'title': 'Low GC All Gen. Mean', 'description': 'Mean Sequencing Depth for All Reads in Low GC-Content Regions'})
        pheader['mu_q_b_b'] = dict(shared_mean, **{'title': 'Low GC Q40 Gen. Mean', 'description': 'Mean Sequencing Depth for Q40 Reads in Low GC-Content Regions'})
        pheader['mu_a_b_t'] = dict(shared_mean, **{'title': 'High GC All Gen. Mean', 'description': 'Mean Sequencing Depth for All Reads in High GC-Content Regions'})
        pheader['mu_q_b_t'] = dict(shared_mean, **{'title': 'High GC Q40 Gen. Mean', 'description': 'Mean Sequencing Depth for Q40 Reads in High GC-Content Regions'})
        pheader['cv_a_b'] = dict(shared_cofv, **{'title': 'All Genome CoV', 'description': 'Sequencing Depth CoV for All Reads'})
        pheader['cv_q_b'] = dict(shared_cofv, **{'title': 'Q40 Genome CoV', 'description': 'Sequencing Depth CoV for Q40 Reads'})
        pheader['cv_a_b_b'] = dict(shared_cofv, **{'title': 'Low GC All Gen. CoV', 'description': 'Sequencing Depth CoV for All Reads in Low GC-Content Regions'})
        pheader['cv_q_b_b'] = dict(shared_cofv, **{'title': 'Low GC Q40 Gen. CoV', 'description': 'Sequencing Depth CoV for Q40 Reads in Low GC-Content Regions'})
        pheader['cv_a_b_t'] = dict(shared_cofv, **{'title': 'High GC All Gen. CoV', 'description': 'Sequencing Depth CoV for All Reads in High GC-Content Regions'})
        pheader['cv_q_b_t'] = dict(shared_cofv, **{'title': 'High GC Q40 Gen. CoV', 'description': 'Sequencing Depth CoV for Q40 Reads in High GC-Content Regions'})

        pheader['mu_a_c'] = dict(shared_mean, **{'title': 'All CpGs Mean', 'description': 'Mean Sequencing Depth for All CpGs'})
        pheader['mu_q_c'] = dict(shared_mean, **{'title': 'Q40 CpGs Mean', 'description': 'Mean Sequencing Depth for Q40 CpGs'})
        pheader['mu_a_c_b'] = dict(shared_mean, **{'title': 'Low GC All CpGs Mean', 'description': 'Mean Sequencing Depth for All CpGs in Low GC-Content Regions'})
        pheader['mu_q_c_b'] = dict(shared_mean, **{'title': 'Low GC Q40 CpGs Mean', 'description': 'Mean Sequencing Depth for Q40 CpGs in Low GC-Content Regions'})
        pheader['mu_a_c_t'] = dict(shared_mean, **{'title': 'High GC All CpGs Mean', 'description': 'Mean Sequencing Depth for All CpGs in High GC-Content Regions'})
        pheader['mu_q_c_t'] = dict(shared_mean, **{'title': 'High GC Q40 CpGs Mean', 'description': 'Mean Sequencing Depth for Q40 CpGs in High GC-Content Regions'})
        pheader['cv_a_c'] = dict(shared_cofv, **{'title': 'All CpGs CoV', 'description': 'Sequencing Depth CoV for All CpGs'})
        pheader['cv_q_c'] = dict(shared_cofv, **{'title': 'Q40 CpGs CoV', 'description': 'Sequencing Depth CoV for Q40 CpGs'})
        pheader['cv_a_c_b'] = dict(shared_cofv, **{'title': 'Low GC All CpGs CoV', 'description': 'Sequencing Depth CoV for All CpGs in Low GC-Content Regions'})
        pheader['cv_q_c_b'] = dict(shared_cofv, **{'title': 'Low GC Q40 CpGs CoV', 'description': 'Sequencing Depth CoV for Q40 CpGs in Low GC-Content Regions'})
        pheader['cv_a_c_t'] = dict(shared_cofv, **{'title': 'High GC All CpGs CoV', 'description': 'Sequencing Depth CoV for All CpGs in High GC-Content Regions'})
        pheader['cv_q_c_t'] = dict(shared_cofv, **{'title': 'High GC Q40 CpGs CoV', 'description': 'Sequencing Depth CoV for Q40 CpGs in High GC-Content Regions'})

        pconfig = {
            'id': 'biscuit_seq_depth',
            'table_title': 'BISCUIT: Sequencing Depth',
            'sortRows': False
        }

        if len(pd) > 0:
            self.add_section(
                name = 'Sequencing Depth Statistics',
                anchor = 'biscuit-seq-depth',
                description = '''
                    Shows the sequence depth mean and uniformity measured by the Coefficient of Variation
                    (`CoV`, defined as `stddev/mean`).
                ''',
                helptext = '''
                    The plot shows coverage across different selections:

                    * _Genome_ (Gen.) - Statistics for all bases across the entire genome
                    * _CpGs_ - Statistics for CpGs
                    * _All_ - Statistics for any mapped bases/CpGs
                    * _Q40_ - Statistics only those bases/CpGs with mapping quality `MAPQ >= 40`
                    * _High GC_ - Bases / CpGs that overlap with the top 10% of 100bp windows for GC-content
                    * _Low GC_ - Bases / CpGs that overlap with the bottom 10% of 100bp windows for GC-content

                ''',
                plot = beeswarm.plot(pd, pheader, pconfig)
            )

    ########################################
    #### Base Coverage and CpG Coverage ####
    ########################################
    def parse_logs_covdist_all_base(self, f, fn):
        '''
        Parses _covdist_all_base_botgc_table.txt
               _covdist_all_base_table.txt
               _covdist_all_base_topgc_table.txt
               _covdist_all_cpg_botgc_table.txt
               _covdist_all_cpg_table.txt
               _covdist_all_cpg_topgc_table.txt
               _covdist_q40_base_botgc_table.txt
               _covdist_q40_base_table.txt
               _covdist_q40_base_topgc_table.txt
               _covdist_q40_cpg_botgc_table.txt
               _covdist_q40_cpg_table.txt
               _covdist_q40_cpg_topgc_table.txt
        Inputs:
            f - current matched file
            fn - filename
        Returns:
            data - dictionary of coverage distributions up to 30X data
        '''
        file_data = f.splitlines()[2:]

        # Handle missing data
        if len(file_data) == 0:
            log.debug('No data available in {}. Will not fill corresponding entries.'.format(fn))
            return dict(zip([i for i in range(31)], [-1 for _ in range(31)]))

        dd = {}
        for l in file_data:
            fields = l.split()
            dd[int(float(fields[0]))] = int(float(fields[1]))

        covs = sorted([k for k in dd])[:31]
        _ccov_cnt = sum(dd.values())

        ccov_cnts = []
        for cov in covs:
            ccov_cnts.append(_ccov_cnt/1000000.0)
            _ccov_cnt -= dd[cov]

        return dict(zip(covs, ccov_cnts))

    def parse_logs_covdist_all_base_botgc(self, f, fn):
        ''' Handled by parse_logs_covdist_all_base() '''
        return self.parse_logs_covdist_all_base(f, fn)

    def parse_logs_covdist_all_base_topgc(self, f, fn):
        ''' Handled by parse_logs_covdist_all_base() '''
        return self.parse_logs_covdist_all_base(f, fn)

    def parse_logs_covdist_q40_base(self, f, fn):
        ''' Handled by parse_logs_covdist_all_base() '''
        return self.parse_logs_covdist_all_base(f, fn)

    def parse_logs_covdist_q40_base_botgc(self, f, fn):
        ''' Handled by parse_logs_covdist_all_base() '''
        return self.parse_logs_covdist_all_base(f, fn)

    def parse_logs_covdist_q40_base_topgc(self, f, fn):
        ''' Handled by parse_logs_covdist_all_base() '''
        return self.parse_logs_covdist_all_base(f, fn)

    def parse_logs_covdist_all_cpg(self, f, fn):
        ''' Handled by parse_logs_covdist_all_base() '''
        return self.parse_logs_covdist_all_base(f, fn)

    def parse_logs_covdist_all_cpg_botgc(self, f, fn):
        ''' Handled by parse_logs_covdist_all_base() '''
        return self.parse_logs_covdist_all_base(f, fn)

    def parse_logs_covdist_all_cpg_topgc(self, f, fn):
        ''' Handled by parse_logs_covdist_all_base() '''
        return self.parse_logs_covdist_all_base(f, fn)

    def parse_logs_covdist_q40_cpg(self, f, fn):
        ''' Handled by parse_logs_covdist_all_base() '''
        return self.parse_logs_covdist_all_base(f, fn)

    def parse_logs_covdist_q40_cpg_botgc(self, f, fn):
        ''' Handled by parse_logs_covdist_all_base() '''
        return self.parse_logs_covdist_all_base(f, fn)

    def parse_logs_covdist_q40_cpg_topgc(self, f, fn):
        ''' Handled by parse_logs_covdist_all_base() '''
        return self.parse_logs_covdist_all_base(f, fn)

    def chart_covdist_all_base(self):
        '''
        Charts _covdist_all_base_botgc_table.txt
               _covdist_all_base_table.txt
               _covdist_all_base_topgc_table.txt
               _covdist_all_cpg_botgc_table.txt
               _covdist_all_cpg_table.txt
               _covdist_all_cpg_topgc_table.txt
               _covdist_q40_base_botgc_table.txt
               _covdist_q40_base_table.txt
               _covdist_q40_base_topgc_table.txt
               _covdist_q40_cpg_botgc_table.txt
               _covdist_q40_cpg_table.txt
               _covdist_q40_cpg_topgc_table.txt
        Inputs:
            No inputs
        Returns:
            No returns, generates Cumulative Coverage chart
        '''

        pd = [
            self.mdata['covdist_all_base'],
            self.mdata['covdist_q40_base'],
            self.mdata['covdist_all_cpg'],
            self.mdata['covdist_q40_cpg'],
            self.mdata['covdist_all_base_botgc'],
            self.mdata['covdist_q40_base_botgc'],
            self.mdata['covdist_all_cpg_botgc'],
            self.mdata['covdist_q40_cpg_botgc'],
            self.mdata['covdist_all_base_topgc'],
            self.mdata['covdist_q40_base_topgc'],
            self.mdata['covdist_all_cpg_topgc'],
            self.mdata['covdist_q40_cpg_topgc']
        ]

        pconfig = {
            'id': 'biscuit_cumulative',
            'title': 'BISCUIT: Cumulative Coverage',
            'ymin': 0,
            'xLabelFormat': '{value}X',
            'tt_label': '<strong>{point.x}X:</strong> {point.y:.2f}M',
            'xlab': 'Coverage',
            'ylab': 'Millions of Bases',
            'data_labels': [
                {'name': 'All Bases', 'ylab': 'Millions of Bases'},
                {'name': 'Q40 Bases', 'ylab': 'Millions of Bases'},
                {'name': 'All CpGs', 'ylab': 'Millions of CpGs'},
                {'name': 'Q40 CpGs', 'ylab': 'Millions of CpGs'},
                {'name': 'Low GC All Bases', 'ylab': 'Millions of Bases'},
                {'name': 'Low GC Q40 Bases', 'ylab': 'Millions of Bases'},
                {'name': 'Low GC All CpGs', 'ylab': 'Millions of CpGs'},
                {'name': 'Low GC Q40 CpGs', 'ylab': 'Millions of CpGs'},
                {'name': 'High GC All Bases', 'ylab': 'Millions of Bases'},
                {'name': 'High GC Q40 Bases', 'ylab': 'Millions of Bases'},
                {'name': 'High GC All CpGs', 'ylab': 'Millions of CpGs'},
                {'name': 'High GC Q40 CpGs', 'ylab': 'Millions of CpGs'}
            ]
        }

        self.add_section(
            name = 'Cumulative Coverage',
            anchor = 'biscuit-cumulative-coverage',
            description = 'Shows the number of bases or CpGs covered by a given number of reads.',
            helptext = '''
                * _All_ - Coverage for any mapped reads
                * _Q40_ - Coverage for reads with mapping quality `MAPQ >= 40`
                * _High GC_ - Coverage for reads that overlap with the top 10% of 100bp windows for GC-content
                * _Low GC_ - Coverage for reads that overlap with the bottom 10% of 100bp windows for GC-content
            ''',
            plot = linegraph.plot(pd, pconfig)
        )

    def chart_covdist_all_base_botgc(self):
        ''' Handled by chart_covdist_all_base() '''
        pass

    def chart_covdist_all_base_topgc(self):
        ''' Handled by chart_covdist_all_base() '''
        pass

    def chart_covdist_q40_base(self):
        ''' Handled by chart_covdist_all_base() '''
        pass

    def chart_covdist_q40_base_botgc(self):
        ''' Handled by chart_covdist_all_base() '''
        pass

    def chart_covdist_q40_base_topgc(self):
        ''' Handled by chart_covdist_all_base() '''
        pass

    def chart_covdist_all_cpg(self):
        ''' Handled by chart_covdist_all_base() '''
        pass

    def chart_covdist_all_cpg_botgc(self):
        ''' Handled by chart_covdist_all_base() '''
        pass

    def chart_covdist_all_cpg_topgc(self):
        ''' Handled by chart_covdist_all_base() '''
        pass

    def chart_covdist_q40_cpg(self):
        ''' Handled by chart_covdist_all_base() '''
        pass

    def chart_covdist_q40_cpg_botgc(self):
        ''' Handled by chart_covdist_all_base() '''
        pass

    def chart_covdist_q40_cpg_topgc(self):
        ''' Handled by chart_covdist_all_base() '''
        pass

    ########################################
    ####          CpG Retention         ####
    ########################################
    def parse_logs_cpg_retention_readpos(self, f, fn):
        '''
        Parses _CpGRetentionByReadPos.txt
               _CpHRetentionByReadPos.txt
        Inputs:
            f - current matched file
            fn - filename
        Returns:
            data - dictionary of fraction of retained cytosines for reads 1 and 2
            in either a CpH or CpG context
        '''
        file_data = f.splitlines()[2:]

        # Handle missing data
        if len(file_data) == 0:
            log.debug('No data available in {}. Will not fill corresponding entries.'.format(fn))
            return {'no_data_available': 1}

        r1 = {'C': {}, 'R': {}}
        r2 = {'C': {}, 'R': {}}
        for l in file_data:
            fields = l.strip().split('\t')

            if fields[0] not in ['1', '2'] or fields[2] not in ['C', 'R']:
                return {}
            if fields[0] == '1':
                r1[fields[2]][int(fields[1])] = int(fields[3])
            elif fields[0] == '2':
                r2[fields[2]][int(fields[1])] = int(fields[3])

        r1rate = OrderedDict()
        for k in sorted(r1['C'].keys()):
            if k in r1['R']:
                r1rate[k] = 100.0 * float(r1['R'][k]) / (r1['R'][k] + r1['C'][k])

        r2rate = OrderedDict()
        for k in sorted(r2['C'].keys()):
            if k in r2['R']:
                r2rate[k] = 100.0 * float(r2['R'][k]) / (r2['R'][k] + r2['C'][k])

        return {'1': r1rate, '2': r2rate}

    def chart_cpg_retention_readpos(self):
        '''
        Charts _CpGRetentionByReadPos.txt
               _CpHRetentionByReadPos.txt
        Inputs:
            No inputs
        Returns:
            No returns, generates Retenion vs. Base Position in Read chart
        '''

        pd = [
            dict([(s_name, dd['1']) for s_name, dd in self.mdata['cpg_retention_readpos'].items() if 'no_data_available' not in dd.keys()]),
            dict([(s_name, dd['2']) for s_name, dd in self.mdata['cpg_retention_readpos'].items() if 'no_data_available' not in dd.keys()]),
            dict([(s_name, dd['1']) for s_name, dd in self.mdata['cph_retention_readpos'].items() if 'no_data_available' not in dd.keys()]),
            dict([(s_name, dd['2']) for s_name, dd in self.mdata['cph_retention_readpos'].items() if 'no_data_available' not in dd.keys()])
        ]

        pconfig = {
            'id': 'biscuit_retention_cytosine',
            'title': 'BISCUIT: Retention vs. Base Position in Read',
            'xlab': 'Position in Read',
            'ylab': 'CpG Retention Rate (%)',
            'ymin': 0,
            'ymax': 100,
            'yMinRange': 0,
            'yFloor': 0,
            'tt_label': '<strong>Position {point.x}:</strong> {point.y:.2f}%',
            'data_labels': [
                {'name': 'CpG Read 1', 'ylab': 'CpG Retention Rate (%)'},
                {'name': 'CpG Read 2', 'ylab': 'CpG Retention Rate (%)'},
                {'name': 'CpH Read 1', 'ylab': 'CpH Retention Rate (%)'},
                {'name': 'CpH Read 2', 'ylab': 'CpH Retention Rate (%)'}
            ]
        }

        self.add_section(
            name = 'Retention vs. Base Position in Read',
            anchor = 'biscuit-retention-cytosine',
            description = 'Distribution of cytosine retention rates across base positions in the read (a.k.a. _M-bias_ plot).',
            plot = linegraph.plot(pd, pconfig)
        )

    def parse_logs_cph_retention_readpos(self, f, fn):
        ''' Handled by parse_logs_cpg_retention_readpos() '''
        return self.parse_logs_cpg_retention_readpos(f, fn)

    def chart_cph_retention_readpos(self):
        ''' Handled by chart_cpg_retention_readpos() '''
        pass

    def parse_logs_read_avg_retention_rate(self, f, fn):
        '''
        Parses _totalReadConversionRate.txt
        Inputs:
            f - current matched file
            fn - filename
        Returns:
            data - dictionary of read averaged fraction of retainied cytosines by context
        '''

        data = {}
        try:
            m = re.match(r'([\d.]+)\t([\d.]+)\t([\d.]+)\t([\d.]+)', f.splitlines()[2])
        except IndexError:
            return {}
        else:
            if m is not None:
                data['rca'] = 100.0 * float(m.group(1))
                data['rcc'] = 100.0 * float(m.group(2))
                data['rcg'] = 100.0 * float(m.group(3))
                data['rct'] = 100.0 * float(m.group(4))

        return data

    def chart_read_avg_retention_rate(self):
        '''
        Charts _totalReadConversionRate.txt
               _totalBaseConversionRate.txt
        Inputs:
            No inputs
        Returns:
            No returns, generates Retenion vs. Base Position in Read chart
        '''

        mdata_byread = {}
        for s_name, dd in self.mdata['read_avg_retention_rate'].items():
            mdata_byread[s_name] = dd

        mdata_bybase = {}
        for s_name, dd in self.mdata['base_avg_retention_rate'].items():
            mdata_bybase[s_name] = dd

        pdata = {}
        for s_name, dd in mdata_byread.items():
            try:
                pdata[s_name] = dict(list(dd.items()) + list(mdata_bybase[s_name].items()))
            except KeyError:
                log.warning("Couldn't find sample when making avg_retention_rate plot: '{}'".format(s_name))

        shared = {
            'format':'{:,.2f}',
            'min':0,
            'max':100,
            'suffix':'%',
            'scale': 'YlGnBu'
        }

        pheader = OrderedDict()
        pheader['rca'] = dict(shared, **{'title':'RA CpA', 'description':'Read Averaged CpA Retention'})
        pheader['rcc'] = dict(shared, **{'title':'RA CpC', 'description':'Read Averaged CpC Retention'})
        pheader['rcg'] = dict(shared, **{'title':'RA CpG', 'description':'Read Averaged CpG Retention'})
        pheader['rct'] = dict(shared, **{'title':'RA CpT', 'description':'Read Averaged CpT Retention'})
        pheader['bca'] = dict(shared, **{'title':'BA CpA', 'description':'Base Averaged CpA Retention'})
        pheader['bcc'] = dict(shared, **{'title':'BA CpC', 'description':'Base Averaged CpC Retention'})
        pheader['bcg'] = dict(shared, **{'title':'BA CpG', 'description':'Base Averaged CpG Retention'})
        pheader['bct'] = dict(shared, **{'title':'BA CpT', 'description':'Base Averaged CpT Retention'})

        pconfig = {
            'id': 'biscuit_retention',
            'table_title': 'BISCUIT: Cytosine Retention',
            'sortRows': False,
            'save_file': True
        }

        self.add_section(
            name = 'Cytosine Retention',
            anchor = 'biscuit-retention',
            description = '''
                Shows the cytosine retention rate for different contexts.
                `RA`: Read-averaged rates.
                `BA`:Base-averaged rates.
            ''',
            helptext = '''
                The cytosine retention rate is calculated as `1 - (cytosine conversion rate)`.

                Assuming complete, but not over, bisulfite conversion, the cytosine retention rate
                is the average cytosine modification (including 5mC, 5hmC, etc) rate.
            ''',
            plot = table.plot(pdata, pheader, pconfig)
        )

    def parse_logs_base_avg_retention_rate(self, f, fn):
        '''
        Parses _totalBaseConversionRate.txt
        Inputs:
            f - current matched file
            fn - filename
        Returns:
            data - dictionary of base averaged fraction of retainied cytosines by context
        '''

        data = {}
        try:
            m = re.match(r'([\d.]+)\t([\d.]+)\t([\d.]+)\t([\d.]+)', f.splitlines()[2])
        except IndexError:
            return {}
        else:
            if m is not None:
                data['bca'] = 100.0 * float(m.group(1))
                data['bcc'] = 100.0 * float(m.group(2))
                data['bcg'] = 100.0 * float(m.group(3))
                data['bct'] = 100.0 * float(m.group(4))

        return data

    def chart_base_avg_retention_rate(self):
        ''' Handled by chart_read_avg_retention_rate() '''
        pass



#!/usr/bin/env python

""" MultiQC module to parse output from mtnucratio """

from __future__ import print_function
from collections import OrderedDict
import logging
import json

from multiqc import config
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """ mtnucratio module """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='mtnucratio', anchor='mtnucratio',
        href="http://www.github.com/apeltzer/MTNucRatioCalculator",
        info="is a tool to compute mt/nuc ratios for NGS datasets.")

        # Find and load any MTNUCRATIO reports
        self.mtnuc_data = dict()

        for f in self.find_log_files('mtnucratio',filehandles=True):
            self.parseJSON(f)

        # Filter to strip out ignored sample names
        self.mtnuc_data = self.ignore_samples(self.mtnuc_data)

        if len(self.mtnuc_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.mtnuc_data)))

        # Write parsed report data to a file
        self.write_data_file(self.mtnuc_data, 'multiqc_mtnucratio')

        # Basic Stats Table
        self.mtnucratio_general_stats_table()


    # Parse our nice little JSON file
    def parseJSON(self, f):
        """ Parse the JSON output from mtnucratio and save the summary statistics """
        try:
            parsed_json = json.load(f['f'])
            if 'metrics' not in parsed_json and 'metadata' not in parsed_json:
                log.warning("No MTNUCRATIO JSON: '{}'".format(f['fn']))
                return None
        except JSONDecodeError as e:
            log.warning("Could not parse mtnucratio JSON: '{}'".format(f['fn']))
            log.debug(e)
            return None

        # Get sample name from JSON first
        s_name = self.clean_s_name(parsed_json['metadata']['sample_name'],'')
        self.add_data_source(f, s_name)

        metrics_dict = parsed_json['metrics']

        # Add all in the main data_table
        self.mtnuc_data[s_name] = metrics_dict


    def mtnucratio_general_stats_table(self):
        """ Take the parsed stats from the mtnucratio report and add it to the
        basic stats table at the top of the report """

        headers = OrderedDict()
        headers['mt_cov_avg'] = {
            'title': 'MT genome coverage',
            'description': 'Average coverage (X) on mitochondrial genome.',
            'min': 0,
            'scale': 'OrRd',
            'suffix': ' X',
            'hidden': True
        }
        headers['nuc_cov_avg'] = {
            'title': 'Genome coverage',
            'description': 'Average coverage (X) on nuclear genome.',
            'min': 0,
            'scale': 'GnBu',
            'suffix': ' X',
            'hidden': True
        }
        headers['mt_nuc_ratio'] = {
            'title': 'MT to Nuclear Ratio',
            'description': 'Mitochondrial to nuclear reads ratio (MTNUC)',
            'min': 0,
            'max': 100,
            'scale': 'RdYlGrn-rev',
        }
        headers['nucreads'] = {
            'title': '{} Genome reads'.format(config.read_count_prefix),
            'description': 'Reads on the nuclear genome ({})'.format(config.read_count_desc),
            'modify': lambda x: x * config.read_count_multiplier,
            'shared_key': 'read_count',
            'scale': 'BuPu',
            'hidden': True
        }
        headers['mtreads'] = {
            'title': '{} MT genome reads'.format(config.read_count_prefix),
            'description': 'Reads on the mitochondrial genome ({})'.format(config.read_count_desc),
            'modify': lambda x: x * config.read_count_multiplier,
            'shared_key': 'read_count',
            'scale': 'OrRd',
            'hidden': True
        }

        self.general_stats_addcols(self.mtnuc_data, headers)



#!/usr/bin/env python

""" MultiQC module to parse output from THetA2 """

from __future__ import print_function
from collections import OrderedDict
import logging

from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='THetA2', anchor='theta2',
        href="http://compbio.cs.brown.edu/projects/theta/",
        info="<em>(Tumor Heterogeneity Analysis)</em> estimates tumour purity "\
             "and clonal / subclonal copy number.")

        # Find and load any THetA2 reports
        self.theta2_data = dict()
        for f in self.find_log_files('theta2', filehandles=True):
            parsed_data = self.parse_theta2_report(f['f'])
            if len(parsed_data) > 0:
                if f['s_name'] in self.theta2_data:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
                self.add_data_source(f)
                self.theta2_data[f['s_name']] = parsed_data

        # Filter to strip out ignored sample names
        self.theta2_data = self.ignore_samples(self.theta2_data)

        if len(self.theta2_data) == 0 :
            raise UserWarning

        log.info("Found {} reports".format(len(self.theta2_data)))

        # Write parsed report data to a file
        self.write_data_file(self.theta2_data, 'multiqc_theta2')

        # Alignment bar plot
        self.add_section (
            name = 'Tumour Subclone Purities',
            anchor = 'theta2-purities',
            description = 'Purities of tumour subclones. <em>NB:</em> Only first maximum likelihood solution for each sample shown.',
            plot = self.theta2_purities_chart()
        )


    def parse_theta2_report (self, fh):
        """ Parse the final THetA2 log file. """
        parsed_data = {}
        for l in fh:
            if l.startswith('#'):
                continue
            else:
                s = l.split("\t")
                purities = s[1].split(',')
                parsed_data['proportion_germline'] = float(purities[0]) * 100.0
                for i, v in enumerate(purities[1:]):
                    if i <= 5:
                        parsed_data['proportion_tumour_{}'.format(i+1)] = float(v) * 100.0
                    else:
                        parsed_data['proportion_tumour_gt5'] = (float(v) * 100.0) + parsed_data.get('proportion_tumour_gt5', 0)
                break
        return parsed_data


    def theta2_purities_chart (self):
        """ Make the plot showing alignment rates """

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['proportion_germline'] =   { 'name': 'Germline' }
        keys['proportion_tumour_1'] =   { 'name': 'Tumour Subclone 1' }
        keys['proportion_tumour_2'] =   { 'name': 'Tumour Subclone 2' }
        keys['proportion_tumour_3'] =   { 'name': 'Tumour Subclone 3' }
        keys['proportion_tumour_4'] =   { 'name': 'Tumour Subclone 4' }
        keys['proportion_tumour_5'] =   { 'name': 'Tumour Subclone 5' }
        keys['proportion_tumour_gt5'] = { 'name': 'Tumour Subclones > 5' }

        # Config for the plot
        pconfig = {
            'id': 'theta2_purity_plot',
            'title': 'THetA2: Tumour Subclone Purities',
            'cpswitch': False,
            'ymin': 0,
            'ymax': 100,
            'ylab': '% Purity',
            'tt_suffix': '%'
        }

        return bargraph.plot(self.theta2_data, keys, pconfig)



#!/usr/bin/env python

""" MultiQC submodule to parse output from RSeQC inner_distance.py
http://rseqc.sourceforge.net/#inner-distance-py """

from collections import OrderedDict
import logging

from multiqc.plots import linegraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """ Find RSeQC inner_distance frequency reports and parse their data """

    # Set up vars
    self.inner_distance = dict()
    self.inner_distance_pct = dict()

    # Go through files and parse data
    for f in self.find_log_files('rseqc/inner_distance'):
        if f['s_name'] in self.inner_distance:
            log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
        self.add_data_source(f, section='inner_distance')
        #saving to temporary variable fro SE checking later
        parsed_data = OrderedDict()
        for l in f['f'].splitlines():
            s = l.split()
            try:
                avg_pos = (float(s[0]) +  float(s[1])) / 2.0
                parsed_data[avg_pos] = float(s[2])
            except:
                # Don't bother running through whole file if wrong
                break

        # Only add if we actually found something i,e it was PE data
        if len(parsed_data) > 0:
            self.inner_distance[f['s_name']] = parsed_data

    # Filter to strip out ignored sample names
    self.inner_distance = self.ignore_samples(self.inner_distance)

    if len(self.inner_distance) > 0:

        # Make a normalised percentage version of the data
        for s_name in self.inner_distance:
            self.inner_distance_pct[s_name] = OrderedDict()
            total = sum( self.inner_distance[s_name].values() )
            for k, v in self.inner_distance[s_name].items():
                self.inner_distance_pct[s_name][k] = (v/total)*100

        # Add line graph to section
        pconfig = {
            'id': 'rseqc_inner_distance_plot',
            'title': 'RSeQC: Inner Distance',
            'ylab': 'Counts',
            'xlab': "Inner Distance (bp)",
            'tt_label': "<strong>{point.x} bp</strong>: {point.y:.2f}",
            'data_labels': [
                {'name': 'Counts', 'ylab': 'Counts'},
                {'name': 'Percentages', 'ylab': 'Percentage'}
            ]
        }
        self.add_section (
            name = 'Inner Distance',
            anchor = 'rseqc-inner_distance',
            description = '<a href="http://rseqc.sourceforge.net/#inner-distance-py" target="_blank">Inner Distance</a>' \
                " calculates the inner distance" \
                " (or insert size) between two paired RNA reads." \
                " Note that this can be negative if fragments overlap.",
            plot = linegraph.plot([self.inner_distance, self.inner_distance_pct], pconfig)
        )

    # Return number of samples found
    return len(self.inner_distance)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" MultiQC module to parse output files from iVar """

from __future__ import print_function
from collections import OrderedDict
import logging
import re

from multiqc import config
from multiqc.plots import bargraph, heatmap
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(
            name = 'iVar',
            anchor = 'iVar',
            href = 'https://github.com/andersen-lab/ivar',
            info = "is a computational package that contains functions broadly useful for viral amplicon-based sequencing."
        )

        # Find and load iVar trim results
        self.ivar_data = dict()
        self.ivar_primers = dict()
        for f in self.find_log_files('ivar/trim', filehandles=True):
            self.parse_ivar(f)

        # Filter to strip out ignored sample names
        self.ivar_data = self.ignore_samples(self.ivar_data)
        self.ivar_primers = self.ignore_samples(self.ivar_primers)

        # Stop if no files are found
        num_samples = max(len(self.ivar_data), len(self.ivar_primers))
        if num_samples == 0:
            raise UserWarning

        # Log number of reports
        log.info("Found {} reports".format(num_samples))

        # Write parsed data to a file
        self.write_data_file(self.ivar_data, 'multiqc_ivar_summary')
        self.write_data_file(self.ivar_primers, 'multiqc_ivar_primers')

        # Basic Stats Table
        self.ivar_general_stats_table()

        # Heatmap info
        self.primer_heatmap()

    # Parse an ivar report
    def parse_ivar(self, f):
        """ Parse iVar log file, either v1.1_beta or v1.2
        """
        count_regexes = {
            'mapped_reads': re.compile(r'(?:Found\s)(\d+)(?:\smapped)'),
            'trimmed_reads': re.compile(r'(?:Trimmed primers from )(?:\d+\.\d+\% \()?(\d+)'),
            'reads_outside_primer_region': re.compile(r'^(?:\d+\.\d+\% \()?(\d+)(?:\))?(?:.*[of]?)reads\s(?:that\s)?started'),
            'reads_too_short_after_trimming': re.compile(r'^(?:\d+\.\d+\% \()?(\d+)(?:\))?(?:.*[of]?)reads\swere(?: quality trimmed | shortened)')
        }
        primer_regex = re.compile(r'^(.*)(?:\t+)(\d+$)')
        parsed_data = dict()
        primers = OrderedDict()
        for l in f['f']:
            # Search count regexes for stats
            for k, count_regex in count_regexes.items():
                count_match = count_regex.search(l)
                if count_match:
                    parsed_data[k] = int(count_match.group(1))

            # Try to match the primer regex
            primer_match = primer_regex.search(l)
            if primer_match:
                primers[ primer_match.group(1) ] = int(primer_match.group(2))

        if parsed_data is not None and len(parsed_data) > 0:
            if f['s_name'] in self.ivar_data:
                log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
            self.ivar_data[f['s_name']] = parsed_data
            self.add_data_source(f, section='trimming')

        if primers is not None and len(primers) > 0:
            self.ivar_primers[f['s_name']] = primers
            self.add_data_source(f, section='trimming')


    # Add to general stats table
    def ivar_general_stats_table(self):
        """ Take the parsed stats from the iVAR report and add it to the
        basic stats table"""

        headers = OrderedDict()
        headers['reads_too_short_after_trimming'] = {
            'title': '{} Too short'.format(config.read_count_prefix),
            'description': 'Number of reads too short (<30bp) after primer trimming ({})'.format(config.read_count_desc),
            'scale': 'OrRd',
            'shared_key': 'read_counts',
            'modify': lambda x: x * config.read_count_multiplier
        }
        headers['reads_outside_primer_region'] = {
            'title': '{} Outside primer'.format(config.read_count_prefix),
            'description': 'Number of reads outside the primer region ({})'.format(config.read_count_desc),
            'scale': 'YlOrBr',
            'shared_key': 'read_counts',
            'modify': lambda x: x * config.read_count_multiplier
        }
        headers['trimmed_reads'] = {
            'title': '{} Primer trimmed'.format(config.read_count_prefix),
            'description': 'Total number of reads where primer trimming was performed. ({})'.format(config.read_count_desc),
            'scale': 'Purples',
            'shared_key': 'read_counts',
            'modify': lambda x: x * config.read_count_multiplier
        }
        headers['mapped_reads'] = {
            'title': '{} Mapped'.format(config.read_count_prefix),
            'description': 'Total number of mapped reads in iVar input. ({})'.format(config.read_count_desc),
            'scale': 'PuBu',
            'shared_key': 'read_counts',
            'modify': lambda x: x * config.read_count_multiplier
        }
        self.general_stats_addcols(self.ivar_data, headers)

    def primer_heatmap(self):
        """ Heatmap showing information on each primer found for every sample """
        # Top level dict contains sample IDs + OrderedDict(primer, counts)

        final_data = list()
        final_xcats = list()
        final_ycats = list()

        for k,v in self.ivar_primers.items():
            final_ycats.append(k)
            tmp_prim_val = list()
            for prim,val in v.items():
                final_xcats.append(prim)
                tmp_prim_val.append(val)
            final_data.append(tmp_prim_val)

        if self.ivar_primers is not None:
            pconfig = {
                'id': 'ivar-primer-count-heatmap',
                'title': 'iVar: Number of primers found for each sample',
                'decimalPlaces': 0,
                'square': False,
                'xcats_samples': False
            }

            self.add_section (
                name = 'iVar Primer Counts',
                anchor = 'ivar-primers-heatmap',
                description ='Counts observed for each primer per sample.',
                helptext = 'This lists the number of times a specific primer was found in the respective sample.',
                plot = heatmap.plot(final_data, final_xcats, final_ycats, pconfig)
            )



#!/usr/bin/env python

""" MultiQC module to parse output from vcftools """

from __future__ import print_function
import logging
from multiqc.modules.base_module import BaseMultiqcModule
from .relatedness2 import Relatedness2Mixin
from .tstv_by_count import TsTvByCountMixin
from .tstv_by_qual import TsTvByQualMixin
from .tstv_summary import TsTvSummaryMixin

# Initialise the logger
log = logging.getLogger(__name__)


class MultiqcModule(BaseMultiqcModule, Relatedness2Mixin, TsTvByCountMixin, TsTvByQualMixin, TsTvSummaryMixin):
    def __init__(self):
        super(MultiqcModule, self).__init__(
            name='VCFTools',
            anchor='vcftools',
            href='https://vcftools.github.io',
            info='is a program for working with and reporting on VCF files.'
        )

        n = dict()
        n['relatedness2'] = self.parse_relatedness2()

        n['tstv_by_count'] = self.parse_tstv_by_count()
        if n['tstv_by_count'] > 0:
            log.info("Found {} TsTv.count reports".format(n['tstv_by_count']))

        n['tstv_by_qual'] = self.parse_tstv_by_qual()
        if n['tstv_by_qual'] > 0:
            log.info("Found {} TsTv.qual reports".format(n['tstv_by_qual']))

        n['tstv_summary'] = self.parse_tstv_summary()
        if n['tstv_summary'] > 0:
            log.info("Found {} TsTv.summary reports".format(n['tstv_summary']))

        if sum(n.values()) == 0:
            raise UserWarning



#!/usr/bin/env python

""" MultiQC functions to plot a scatter plot """

import logging
import random

from multiqc.utils import config, report

logger = logging.getLogger(__name__)

letters = 'abcdefghijklmnopqrstuvwxyz'

def plot (data, pconfig=None):
    """ Plot a scatter plot with X,Y data.
    :param data: 2D dict, first keys as sample names, then x:y data pairs
    :param pconfig: optional dict with config key:value pairs. See CONTRIBUTING.md
    :return: HTML and JS, ready to be inserted into the page
    """
    if pconfig is None:
        pconfig = {}

    # Allow user to overwrite any given config for this plot
    if 'id' in pconfig and pconfig['id'] and pconfig['id'] in config.custom_plot_config:
        for k, v in config.custom_plot_config[pconfig['id']].items():
            pconfig[k] = v

    # Given one dataset - turn it into a list
    if type(data) is not list:
        data = [data]

    # Generate the data dict structure expected by HighCharts series
    plotdata = list()
    for data_index, ds in enumerate(data):
        d = list()
        for s_name in ds:
            # Ensure any overwritting conditionals from data_labels (e.g. ymax) are taken in consideration
            series_config = pconfig.copy()
            if 'data_labels' in pconfig and type(pconfig['data_labels'][data_index]) is dict:  # if not a dict: only dataset name is provided
                series_config.update(pconfig['data_labels'][data_index])

            if type(ds[s_name]) is not list:
                ds[s_name] = [ ds[s_name] ]
            for k in ds[s_name]:
                if k['x'] is not None:
                    if 'xmax' in series_config and float(k['x']) > float(series_config['xmax']):
                        continue
                    if 'xmin' in series_config and float(k['x']) < float(series_config['xmin']):
                        continue
                if k['y'] is not None:
                    if 'ymax' in series_config and float(k['y']) > float(series_config['ymax']):
                        continue
                    if 'ymin' in series_config and float(k['y']) < float(series_config['ymin']):
                        continue
                this_series = { 'x': k['x'], 'y': k['y'] }
                try:
                    this_series['name'] = "{}: {}".format(s_name, k['name'])
                except KeyError:
                    this_series['name'] = s_name
                try:
                    this_series['color'] = k['color']
                except KeyError:
                    try:
                        this_series['color'] = series_config['colors'][s_name]
                    except KeyError:
                        pass
                d.append(this_series)
        plotdata.append(d)

    # Add on annotation data series
    try:
        if pconfig.get('extra_series'):
            extra_series = pconfig['extra_series']
            if type(pconfig['extra_series']) == dict:
                extra_series = [[ pconfig['extra_series'] ]]
            elif type(pconfig['extra_series']) == list and type(pconfig['extra_series'][0]) == dict:
                extra_series = [ pconfig['extra_series'] ]
            for i, es in enumerate(extra_series):
                for s in es:
                    plotdata[i].append(s)
    except (KeyError, IndexError):
        pass

    # Make a plot
    return highcharts_scatter_plot(plotdata, pconfig)

def highcharts_scatter_plot (plotdata, pconfig=None):
    """
    Build the HTML needed for a HighCharts scatter plot. Should be
    called by scatter.plot(), which properly formats input data.
    """
    if pconfig is None:
        pconfig = {}

    # Get the plot ID
    if pconfig.get('id') is None:
        pconfig['id'] = 'mqc_hcplot_'+''.join(random.sample(letters, 10))

    # Sanitise plot ID and check for duplicates
    pconfig['id'] = report.save_htmlid(pconfig['id'])

    # Build the HTML for the page
    html = '<div class="mqc_hcplot_plotgroup">'

    # Buttons to cycle through different datasets
    if len(plotdata) > 1:
        html += '<div class="btn-group hc_switch_group">\n'
        for k, p in enumerate(plotdata):
            active = 'active' if k == 0 else ''
            try:
                name = pconfig['data_labels'][k]['name']
            except:
                name = k+1
            try:
                ylab = 'data-ylab="{}"'.format(pconfig['data_labels'][k]['ylab'])
            except:
                ylab = 'data-ylab="{}"'.format(name) if name != k+1 else ''
            try:
                ymax = 'data-ymax="{}"'.format(pconfig['data_labels'][k]['ymax'])
            except:
                ymax = ''
            try:
                xlab = 'data-xlab="{}"'.format(pconfig['data_labels'][k]['xlab'])
            except:
                xlab = 'data-xlab="{}"'.format(name) if name != k+1 else ''
            html += '<button class="btn btn-default btn-sm {a}" data-action="set_data" {y} {ym} {xl} data-newdata="{k}" data-target="{id}">{n}</button>\n'.format(a=active, id=pconfig['id'], n=name, y=ylab, ym=ymax, xl=xlab, k=k)
        html += '</div>\n\n'

    # The plot div
    html += '<div class="hc-plot-wrapper"><div id="{id}" class="hc-plot not_rendered hc-scatter-plot"><small>loading..</small></div></div></div> \n'.format(id=pconfig['id'])

    report.num_hc_plots += 1

    report.plot_data[pconfig['id']] = {
        'plot_type': "scatter",
        'datasets': plotdata,
        'config': pconfig
    }

    return html
